@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.132 2014/02/18 17:01:46 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.138 2016/03/12 17:48:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -63,11 +63,9 @@ long DVD::GetCapacityValue() {
 }
 
 /** Calculates video bitrate if it set to auto */
-wxArrayInt DVD::CalculateVideoBitrate() {
-	long size = 0;
-	long fixSize = s_config.GetDvdReservedSpace();
-	long fixDuration = 0;
-	long duration = 0;
+void DVD::CalculateVideoBitrate(long& encSize, long& encDuration, long& fixSize, long& fixDuration, long& fixAvgBitrate) {
+	fixSize = s_config.GetDvdReservedSpace();
+	fixDuration = 0;
 	if (m_videoBitrateAuto)
 		SetVideoBitrate(500);
 	// menus
@@ -82,6 +80,10 @@ wxArrayInt DVD::CalculateVideoBitrate() {
 		}
 	}
 	// titles
+	encSize = 0;
+	encDuration = 0;
+	long titlesFixSize = 0;
+	long titlesFixDuration = 0;
 	for (int tsi = 0; tsi < (int) GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, false);
 		for (int pgci = 0; pgci < (int) pgcs.Count(); pgci++) {
@@ -99,32 +101,35 @@ wxArrayInt DVD::CalculateVideoBitrate() {
 					}
 				}
 				if (videoFormat != vfCOPY) {
-					duration += lround(vob->GetDuration());
-					size += vob->GetSize(this);
-				} else { 
-					fixDuration += lround(vob->GetDuration());
-					fixSize += vob->GetSize(this);
+					encSize += vob->GetSize(this);
+					encDuration += lround(vob->GetResultDuration());
+				} else {
+					long size = vob->GetSize(this);
+					long dur = lround(vob->GetResultDuration());
+					fixSize += size;
+					fixDuration += dur;
+					titlesFixSize += vob->GetSlideshow() ?
+							vob->GetSlideshow()->GetResultDuration()*s_config.GetSlideshowVideoBitrate()/8 : size;
+					titlesFixDuration += dur;
 				}
 			}
 		}
 	}
+	
 	long bitrate = GetVideoBitrate();
 	if (m_videoBitrateAuto) {
 		if (GetCapacity() != dcUNLIMITED) {
 			// size - size of video by bitrate 500 + size of audio streams
-			long s = GetCapacityValue() - size - fixSize;
-			bitrate = 500 + (s > 0 && duration > 0 ? s/duration*8 : (duration > 0 ? 0 : 7500));
+			long s = GetCapacityValue() - encSize - fixSize;
+			bitrate = 500 + (s > 0 && encDuration > 0 ? s/encDuration*8 : (encDuration > 0 ? 0 : 7500));
 			if (bitrate > 8000)
 				bitrate = 8000;
 		} else
 			bitrate = 4500;
+		encSize = bitrate*encDuration/8;
 		SetVideoBitrate(bitrate);
 	}
-	wxArrayInt res;
-	res.Add(bitrate);
-	res.Add(fixSize);
-	res.Add(fixDuration);
-	return res;
+	fixAvgBitrate = titlesFixDuration > 0 ? titlesFixSize/titlesFixDuration*8 : 0;
 }
 
 unsigned int DVD::GetAudioStreamCount() {
@@ -149,6 +154,8 @@ unsigned int DVD::GetSubtitleStreamsCount() {
  **/
 int DVD::AddTitleset() {
 	Titleset* titleset = new Titleset;
+	titleset->GetMenus().SetAspectRatio(m_aspectRatio);
+	titleset->GetMenus().SetVideoFormat(m_videoFormat);
 	GetTitlesets().Add(titleset);
 	return GetTitlesets().Count();
 }
@@ -270,8 +277,7 @@ bool DVD::CheckMenus() {
 					return false;
 				}
 				// check if buttons overlapping
-				if (menu->isButtonsOverlapping()) {
-					wxLogError(src + _("Some buttons are overlapping"));
+				if (menu->isButtonsOverlapping(src)) {
 					return false;
 				}
 			}
@@ -283,6 +289,8 @@ bool DVD::CheckMenus() {
 			pgcs[0]->GetEntries().insert(wxT("title"));
 		}
 	}
+	if (!IsEmptyMenuEnabled() && HasCallLastMenu())
+		EnableEmptyMenu();
 	return true;
 }
 
@@ -367,9 +375,9 @@ void DVD::UpdateMenusLocations() {
 /** Updates image in buttons with given jump action */
 bool DVD::UpdateButtonImageFor(int actionTsi, int actionPgci) {
 	bool result = false;
-	for (int tsi = -1; tsi < (int) GetTitlesets().GetCount(); tsi++) {
+	for (int tsi = -1; tsi < (int) GetTitlesets().size(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
-		for (unsigned int pgci = 0; pgci < pgcs.GetCount(); pgci++) {
+		for (unsigned int pgci = 0; pgci < pgcs.size(); pgci++) {
 			Menu* menu = pgcs[pgci]->GetMenu();
 			if (menu)
 				result |= menu->UpdateButtonImageFor(actionTsi != tsi ? actionTsi : -2, actionPgci, this);
@@ -381,9 +389,9 @@ bool DVD::UpdateButtonImageFor(int actionTsi, int actionPgci) {
 
 /** Fix coordinates of buttons if they are out of range */
 void DVD::FixButtonCoordinates() {
-	for (int tsi = -1; tsi < (int) GetTitlesets().GetCount(); tsi++) {
+	for (int tsi = -1; tsi < (int) GetTitlesets().size(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
-		for (unsigned int pgci = 0; pgci < pgcs.GetCount(); pgci++) {
+		for (unsigned int pgci = 0; pgci < pgcs.size(); pgci++) {
 			Menu* menu = pgcs[pgci]->GetMenu();
 			if (menu)
 				menu->FixButtonCoordinates();
@@ -392,13 +400,24 @@ void DVD::FixButtonCoordinates() {
 }
 
 
-/** Sets video format of DVD */
-void DVD::SetVideoFormat(VideoFormat videoFormat, bool updateMenus) {
-	m_videoFormat = videoFormat;
-	if (updateMenus) {
-		for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
-			GetPgcArray(tsi, true).SetVideoFormat(videoFormat);
+/** Returns true if button with given jump action exits */
+bool DVD::HasButtonWithJumpAction(int actionTsi, int actionPgci, bool isMenu) {
+	for (int tsi = -1; tsi < (int) GetTitlesets().size(); tsi++) {
+		PgcArray& pgcs = GetPgcArray(tsi, true);
+		for (unsigned int pgci = 0; pgci < pgcs.size(); pgci++) {
+			Menu* menu = pgcs[pgci]->GetMenu();
+			if (menu && menu->GetButtonByJumpAction(actionTsi != tsi ? actionTsi : -2, actionPgci, isMenu) != NULL)
+				return true;
+		}
 	}
+	return false;
+}
+
+/** Sets video format of DVD */
+void DVD::SetVideoFormat(VideoFormat videoFormat) {
+	m_videoFormat = videoFormat;
+	for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
+		GetPgcArray(tsi, true).SetVideoFormat(videoFormat);
 }
 
 /** Sets audio format of DVD */
@@ -407,12 +426,11 @@ void DVD::SetAudioFormat(AudioFormat format) {
 }
 
 /** Sets aspect ratio of DVD */
-void DVD::SetAspectRatio(AspectRatio aspectRatio, bool updateMenus) {
+void DVD::SetAspectRatio(AspectRatio aspectRatio) {
 	m_aspectRatio = aspectRatio;
-	if (updateMenus) {
-		for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
-			GetPgcArray(tsi, true).SetAspectRatio(aspectRatio);
-	}
+	m_vmgm.SetAspectRatio(m_aspectRatio);
+	for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
+		GetPgcArray(tsi, true).SetAspectRatio(aspectRatio);
 }
 
 /** Returns default title post command */
@@ -766,7 +784,7 @@ long DVD::GetRequiredSize(Cache* cache) {
 	return size;
 }
 
-/** Returns true if at least one button has checked flag "Play All" */
+/** Returns true if at least one button has checked flag "Play all" */
 bool DVD::HasPlayAllButton() {
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
@@ -787,6 +805,28 @@ bool DVD::HasPlayAllButton() {
 	return false;
 }
 
+/** Returns true if at least one button has checked flag "Play all titlesets" */
+bool DVD::HasPlayAllTitlesetButton() {
+	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
+		PgcArray& pgcs = GetPgcArray(tsi, true);
+		for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++) {
+			Pgc* pgc = pgcs[pgci];
+			for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++) {
+				Vob* vob = pgc->GetVobs()[vobi];
+				if (vob->GetMenu()) {
+					Menu* menu = vob->GetMenu();
+					for (unsigned int obji = 0; obji<menu->GetObjectsCount(); obji++)
+						if (menu->GetObject(obji)->IsButton()
+								&& menu->GetObject(obji)->GetAction().IsPlayAll()
+								&& menu->GetObject(obji)->GetAction().IsPlayAllTitlesets())
+							return true;
+				}
+			}
+		}
+	}
+	return false;
+}
+
 /** Returns true if at least one menu has checked flag "Remember last selected button" */
 bool DVD::HasRememberLastButton(bool checkIfNeedRegister) {
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
@@ -869,7 +909,7 @@ bool DVD::IsRegisterUsed(int g) {
 wxArrayString DVD::GetVideoFormatLabels(bool copy, bool none, bool menu) {
 	wxArrayString formats;
 	if (none)
-		formats.Add(_("None"));
+		formats.Add(_("Omit"));
 	if (copy)
 		formats.Add(_("Copy"));
 	formats.Add(wxT("PAL 720x576"));
@@ -888,7 +928,7 @@ wxArrayString DVD::GetVideoFormatLabels(bool copy, bool none, bool menu) {
 wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none, bool pcm) {
 	wxArrayString formats;
 	if (none)
-		formats.Add(_("None"));
+		formats.Add(_("Omit"));
 	if (copy)
 		formats.Add(_("Copy"));
 	formats.Add(_("MP2 48 kHz"));
@@ -901,7 +941,7 @@ wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none, bool pcm) {
 wxArrayString DVD::GetSubtitleFormatLabels(bool copy, bool none) {
 	wxArrayString formats;
 	if (none)
-		formats.Add(_("None"));
+		formats.Add(_("Omit"));
 	if (copy)
 		formats.Add(_("Copy"));
 	return formats;
