@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: DVD.h,v 1.84 2014/01/28 18:02:14 ntalex Exp $
+// RCS-ID:      $Id: DVD.h,v 1.89 2016/03/12 17:48:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -54,7 +54,7 @@ public:
     /** Sets video bitrate */
     void SetVideoBitrate(int bitrate) { m_videoBitrate = bitrate; }
     /** Calculates video bitrate if it set to auto */
-    wxArrayInt CalculateVideoBitrate();
+    void CalculateVideoBitrate(long& encSize, long& encDuration, long& fixSize, long& fixDuration, long& fixAvgBitrate);
 	/** Returns audio bitrate */
     int GetAudioBitrate() { return m_audioBitrate; }
     /** Sets audio bitrate */
@@ -122,9 +122,11 @@ public:
 	bool UpdateButtonImageFor(int actionTsi, int actionPgci);
 	/** Fix coordinates of buttons if they are out of range */
 	void FixButtonCoordinates();
+	/** Returns true if button with given jump action exits */
+	bool HasButtonWithJumpAction(int tsi, int pgci, bool isMenu = false);
 	
 	/** returns id for specified vob */
-	static inline int MakeId(int tsi, int pgci, int vobi, bool menu)
+	static inline int MakeId(int tsi, int pgci, int vobi, bool menu = false)
 	{ return ((tsi+1)<<24) + ((pgci*2+(menu ? 0 : 1))<<8) + vobi; }
 	static inline int GetTsi(int id)  { return (id>>24)-1; }
 	static inline int GetPgci(int id) { return ((id>>8) & 0xFFFF) / 2; }
@@ -144,11 +146,13 @@ public:
 	inline int GetLastMenuRegister() { return m_lastMenuRegister; }
 	/** Returns true if at least one title use "call last menu" command */
 	bool HasCallLastMenu();
+	/** Returns true if at least one button has checked flag "Play all titlesets" */
+	bool HasPlayAllTitlesetButton();
 
 	/** Returns video format of DVD */
 	inline VideoFormat GetVideoFormat() { return m_videoFormat; }
 	/** Sets video format of DVD */
-	void SetVideoFormat(VideoFormat format, bool updateMenus = false);
+	void SetVideoFormat(VideoFormat format);
 	/** Returns audio format of DVD */
 	inline AudioFormat GetAudioFormat() { return m_audioFormat; }
 	/** Sets audio format of DVD */
@@ -156,7 +160,7 @@ public:
 	/** Returns aspect ratio of DVD */
 	inline AspectRatio GetAspectRatio() { return m_aspectRatio; }
 	/** Sets aspect ratio of DVD */
-	void SetAspectRatio(AspectRatio format, bool updateMenus = false);
+	void SetAspectRatio(AspectRatio format);
     /** Returns default title post command */
     DefaultPostCommand GetDefPostCommand() { return m_defPostCommand; }
     /** Returns default title post command as string */
@@ -246,7 +250,7 @@ private:
 	AudioFormat m_audioFormat;
 	AspectRatio m_aspectRatio;
 	wxSvgXmlNode* titleMenuTemplate;
-	/** Returns true if at least one button has checked flag "Play All" */
+	/** Returns true if at least one button has checked flag "Play all" */
 	bool HasPlayAllButton();
 	/** Returns true if at least one menu has checked flag "Remember last selected button" */
 	bool HasRememberLastButton(bool checkIfNeedRegister = false);
