@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.189 2014/09/27 06:01:55 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.193 2016/05/11 20:08:50 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -558,7 +558,7 @@ void MainWin::OnTitleInfo(wxCommandEvent& event) {
 				DVD::GetPgci(id), DVD::GetVobi(id));
 		if (vob) {
 			text = vob->GetFilenameDisplay();
-			double duration = vob->GetDuration();
+			double duration = vob->GetResultDuration();
 			if (vob->GetSlideshow()) {
 				text = wxString::Format(_("%d slides"), vob->GetSlideshow()->GetCount());
 				duration = vob->GetSlideshow()->GetResultDuration();
@@ -686,7 +686,9 @@ void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLab
 		m_dvd->SetAudioFormat(audioFormat);
 		m_dvd->SetAspectRatio(aspectRatio);
 		m_dvd->SetDefPostCommand(defPostCommand);
-		if (m_dvd->GetTitlesets().size() && m_dvd->GetTitlesets()[0]->GetMenus().size()) {
+		if (m_dvd->GetTitlesets().size() == 0)
+			m_dvd->AddTitleset();
+		if (m_dvd->GetTitlesets()[0]->GetMenus().size()) {
 			// move menus to VMG
 			Menus& menus = m_dvd->GetTitlesets()[0]->GetMenus();
 			Menus& vmMenus = m_dvd->GetVmgm();
@@ -998,6 +1000,7 @@ void MainWin::OnBurn(wxCommandEvent& event) {
  */
 void MainWin::Burn(bool autoStart) {
 	// check if titles fit on DVD
+	m_titlesetManager->AskForReencoding(m_dvd->GetVideoFormat());
 	if (m_dvd->GetCapacity() != dcUNLIMITED && m_dvd->GetSize() > m_dvd->GetCapacityValue()) {
 		wxMessageBox(_("The selected video files don't fit on DVD. Please reduce\n\
 video quality (bitrate) or remove some titles."), _("Burn"), wxOK|wxICON_ERROR, this);
@@ -1247,7 +1250,7 @@ void MainWin::OnAddChapterMenu(wxCommandEvent& event) {
 }
 
 void MainWin::OnAddChapterMenuUpdateUI(wxUpdateUIEvent& event) {
-	event.Enable(m_dvd->GetTitleMenuTemplate() != NULL);
+	event.Enable(m_dvd->GetTitleMenuTemplate() != NULL && m_dvd->GetTitlesets().size() > 0);
 }
 
 void MainWin::OnAddVmMenu(wxCommandEvent& event) {
@@ -1273,7 +1276,7 @@ void MainWin::OnDVDOptions(wxCommandEvent& event) {
 }
 
 void MainWin::OnSettings(wxCommandEvent& event) {
-	SettingsDlg dlg(this);
+	SettingsDlg dlg(this, &m_cache);
 	dlg.ShowModal();
 }
 
