@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.123 2014/04/29 20:06:34 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.127 2016/12/28 20:10:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +13,7 @@
 #include "Palette3D.h"
 #include "DVD.h"
 #include "Config.h"
+#include "Utils.h"
 #include <wxVillaLib/SConv.h>
 #include <wxVillaLib/utils.h>
 #include <wxSVG/svg.h>
@@ -299,9 +300,14 @@ void Menu::RemoveChangeable(wxSVGElement* element, MenuObject* obj) {
 			bool isChangeable = false;
 			for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 				MenuObjectParam* param = obj->GetObjectParam(i);
-				if (param->element == elem->GetId() && param->isChangeable()) {
-					elem->SetAttribute(param->attribute, wxT("none"));
-					isChangeable = true;
+				if (!param->isChangeable())
+					continue;
+				for (vector<wxString>::const_iterator elemIt = param->element.begin();
+						elemIt != param->element.end(); elemIt++) {
+					if (*elemIt == elem->GetId()) {
+						elem->SetAttribute(param->attribute, wxT("none"));
+						isChangeable = true;
+					}
 				}
 			}
 			if (isChangeable)
@@ -330,12 +336,15 @@ bool Menu::RemoveNotChangeable(wxSVGElement* element, MenuObject* obj) {
 			bool isChangeable = false;
 			for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 				MenuObjectParam* param = obj->GetObjectParam(i);
-				if (param->element == elem->GetId() && param->changeable) {
-					if (param->normalColour == param->highlightedColour
-							&& param->normalColour == param->selectedColour)
-						elem->SetAttribute(param->attribute, wxT("none"));
-					else
-						isChangeable = true;
+				for (vector<wxString>::const_iterator elemIt = param->element.begin();
+						elemIt != param->element.end(); elemIt++) {
+					if (*elemIt == elem->GetId() && param->changeable) {
+						if (param->normalColour == param->highlightedColour
+								&& param->normalColour == param->selectedColour)
+							elem->SetAttribute(param->attribute, wxT("none"));
+						else
+							isChangeable = true;
+					}
 				}
 			}
 			if (isChangeable) {
@@ -425,11 +434,14 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 				for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 					MenuObjectParam* param = obj->GetObjectParam(i);
 					if (param->isChangeable() && drawType != mdBUTTONS_NORMAL) {
-						wxSVGElement* elem = (wxSVGElement*) symbol->GetElementById(param->element);
-						if (elem && param->attribute.length()) {
-							wxSVGPaint paint(drawType == mdBUTTONS_SELECTED ? param->selectedColour
-									: param->highlightedColour);
-							elem->SetAttribute(param->attribute, paint.GetCSSText());
+						for (vector<wxString>::const_iterator elemIt = param->element.begin();
+								elemIt != param->element.end(); elemIt++) {
+							wxSVGElement* elem = (wxSVGElement*) symbol->GetElementById(*elemIt);
+							if (elem && param->attribute.length()) {
+								wxSVGPaint paint(drawType == mdBUTTONS_SELECTED ? param->selectedColour
+										: param->highlightedColour);
+								elem->SetAttribute(param->attribute, paint.GetCSSText());
+							}
 						}
 					}
 				}
@@ -736,14 +748,19 @@ bool Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
 			if (pgc != NULL) {
 				if (isUpdateButton) {
 					uri = pgc->GetVideoFrameUri(action.GetChapter());
-					uri.AfterLast(wxT('#')).ToLong(&pos);
-					uri = uri.Find(wxT('#')) >= 0 ? uri.BeforeLast(wxT('#')) : uri;
+					if (uri.Find(wxT('#')) >= 0) {
+						uri.AfterLast(wxT('#')).ToLong(&pos);
+						uri = uri.BeforeLast(wxT('#'));
+					}
 				} else {
 					uri = pgc->GetVideoFrameUri(-1, -1);
 				}
 			}
-			if (!obj->IsCustomVideoFrame() || uri.length() == 0
-					|| uri != obj->GetParam(param->name).BeforeLast(wxT('#'))) {
+			wxString objUri = obj->GetParam(param->name);
+			objUri = objUri.Find(wxT('#')) >= 0 ? objUri.BeforeLast(wxT('#')) : objUri;
+			
+			// update image if custom video frame isn't selected and filename isn't changed
+			if (!obj->IsCustomVideoFrame() || uri.length() == 0 || uri != objUri) {
 				double dur = obj->GetParamVideoDuration(param->name);
 				obj->SetParamImageVideo(param->name, uri, pos, (int) lround(dur));
 				obj->UpdateSize();
@@ -819,9 +836,12 @@ bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wx
 	spuNode->AddProperty(_T("highlight"), hlFile);
 	spuNode->AddProperty(_T("select"), selFile);
 	spuNode->AddProperty(_T("force"), _T("yes"));
+	if (s_config.GetButtonsOffset2px()) {
+		spuNode->AddProperty(_T("xoffset"), _T("2"));
+		spuNode->AddProperty(_T("yoffset"), _T("2"));
+	}
 	// buttons
-	for (int i=0; i<(int)GetObjectsCount(); i++)
-	{
+	for (int i=0; i<(int)GetObjectsCount(); i++) {
 		MenuObject* obj = GetObject(i);
 		if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true, obj->GetId(), false))
 			spuNode->AddChild(obj->GetXML(SPUMUX_XML, m_dvd, mode));
