@@ -3,13 +3,14 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.81 2015/01/25 18:17:53 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.88 2016/06/20 10:42:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MenuObjectPropDlg.h"
 #include "VideoFrameDlg.h"
+#include "AnimationDlg.h"
 #include "Config.h"
 #include <wxVillaLib/utils.h>
 #include <wxSVG/svg.h>
@@ -40,6 +41,7 @@ enum {
 	JUSTIFY_LEFT_BT_ID,
 	JUSTIFY_CENTER_BT_ID,
 	JUSTIFY_RIGHT_BT_ID,
+	ANIMATIONS_BT_ID,
 };
 
 BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
@@ -54,6 +56,7 @@ BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
 	EVT_TOGGLEBUTTON(JUSTIFY_LEFT_BT_ID, MenuObjectPropDlg::OnJustifyText)
 	EVT_TOGGLEBUTTON(JUSTIFY_CENTER_BT_ID, MenuObjectPropDlg::OnJustifyText)
 	EVT_TOGGLEBUTTON(JUSTIFY_RIGHT_BT_ID, MenuObjectPropDlg::OnJustifyText)
+	EVT_BUTTON(ANIMATIONS_BT_ID, MenuObjectPropDlg::OnAnimationsBt)
 END_EVENT_TABLE()
 
 MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, bool multObjects, Menu* menu, DVD* dvd, int tsi,
@@ -87,19 +90,26 @@ MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, bool multObj
 
 void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 	wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
-	wxFlexGridSizer* grid = NULL;
-
+	wxBoxSizer* paneSz = mainSizer;
+#if wxUSE_COLLPANE
+	wxCollapsiblePane* collpane = NULL;
+#endif
+	
 	// action
 	if (m_object->IsButton() && !m_multObjects) {
 		DVDAction& action = m_object->GetAction();
+#if wxUSE_COLLPANE && defined(__WXMSW__)
+		collpane = new wxCollapsiblePane(this, wxID_ANY, _("Action"));
+		collpane->GetControlWidget()->SetFont(wxFont(collpane->GetControlWidget()->GetFont().GetPointSize() + 2,
+				wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD));
+		mainSizer->Add(collpane, 0, wxEXPAND);
+		paneSz = new wxBoxSizer(wxVERTICAL);
+		collpane->GetPane()->SetSizer(paneSz);
+		SetPropWindow(collpane->GetPane());
+#else
 		AddStaticLine(mainSizer, _("Action"));
-//		wxCollapsiblePane* collPane = new wxCollapsiblePane(this, wxID_ANY, _("Action"));
-//		mainSizer->Add(collPane, 0, wxEXPAND);
-//#ifdef __WXMSW__
-//		((wxButton*) ((wxGenericCollapsiblePane*) collPane)->GetControlWidget())->SetWindowStyle(wxBORDER_NONE | wxBU_NOTEXT);
-//		((wxButton*) ((wxGenericCollapsiblePane*) collPane)->GetControlWidget())->SetLabel("test");
-//#endif
-		grid = new wxFlexGridSizer(2, 4, 16);
+#endif
+		wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
 		grid->AddGrowableCol(1);
 		// check menu entry if it is set
 		wxString customAction = action.GetCustom();
@@ -146,7 +156,6 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 				CUSTOM_ACTION_RADIO_ID);
 		AddTextProp(grid, wxT(""), customAction);
 		m_customActionEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
-		mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 4);
 		wxCommandEvent evt;
 		if (customAction.length())
 			OnCustomActionSelected(evt);
@@ -174,41 +183,107 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		AddChoiceProp(s, wxT(""), labels[subtitleIdx], labels, 0, true);
 		if (m_dvd->GetSubtitleStreamsCount() == 0)
 			GetLastControl()->Enable(false);
-		
-		AddCheckProp(mainSizer, _("Auto execute command by select"), m_object->IsAutoExecute());
+
+		paneSz->Add(grid, 0, wxEXPAND|wxBOTTOM, 4);
+		AddCheckProp(paneSz, _("Auto execute command by select"), m_object->IsAutoExecute());
 
 		grid = new wxFlexGridSizer(3, 4, 16);
 		AddText(grid, _("Navigation"));
-		AddNavigationProp(grid, nbUP);
+		CreateNavigationProp(grid, nbUP);
 		grid->Add(10,10);
-		AddNavigationProp(grid, nbLEFT);
+		CreateNavigationProp(grid, nbLEFT);
 		grid->Add(10,10);
-		AddNavigationProp(grid, nbRIGHT);
+		CreateNavigationProp(grid, nbRIGHT);
 		grid->Add(10,10);
-		AddNavigationProp(grid, nbDOWN);
+		CreateNavigationProp(grid, nbDOWN);
 		grid->Add(10,10);
-		mainSizer->Add(grid, 0, wxEXPAND|wxTOP|wxBOTTOM, 4);
+		paneSz->Add(grid, 0, wxEXPAND|wxTOP|wxBOTTOM, 4);
+		
+#if wxUSE_COLLPANE && defined(__WXMSW__)
+		paneSz->SetSizeHints(collpane->GetPane());
+		collpane->Expand();
+		
+		collpane = new wxCollapsiblePane(this, wxID_ANY, _("Look"));
+		collpane->GetControlWidget()->SetFont(wxFont(collpane->GetControlWidget()->GetFont().GetPointSize() + 2,
+				wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD));
+		mainSizer->Add(collpane, 0, wxEXPAND);
+		paneSz = new wxBoxSizer(wxVERTICAL);
+		collpane->GetPane()->SetSizer(paneSz);
+		SetPropWindow(collpane->GetPane());
+#else
 		AddStaticLine(mainSizer, _("Look"));
+#endif
 	}
-    CreateLook(mainSizer, grid);
+    CreateLook(paneSz);
     if (!m_multObjects)
-		CreateLocAndSize(mainSizer);
-
+		CreateLocAndSize(paneSz);
+    
+#if wxUSE_COLLPANE
+    if (collpane) {
+		paneSz->SetSizeHints(collpane->GetPane());
+		collpane->Expand();
+		SetPropWindow(this);
+    }
+#endif
 	sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 4);
 }
 
-void MenuObjectPropDlg::CreateLook(wxBoxSizer* mainSizer, wxFlexGridSizer* grid) {
+void MenuObjectPropDlg::CreateNavigationProp(wxSizer* sizer, NavigationButton navButton) {
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
+	AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString());
+	wxChoice* ctrl = (wxChoice*) m_controls[m_controls.Count()-1];
+	
+	// default navigation
+	wxString defBtId = m_object->GetDefaultFocusDest(navButton);
+	wxString text = _("none");
+	if (defBtId != m_object->GetId()) {
+		long l = -1;
+		if (defBtId.Mid(6).ToLong(&l))
+			text = _("button") + wxString::Format(wxT(" %d"), (int) l);
+	}
+	ctrl->Append(_("auto") + wxString(wxT(" (")) + text + wxT(")"), new wxString);
+	ctrl->Append(_("none"), new wxString(m_object->GetId()));
+
+	// add all buttons except current
+	int sel = m_object->GetFocusDest(navButton) == m_object->GetId() ? 1 : 0;
+	for (unsigned int i=0; i<m_menu->GetObjectsCount(); i++) {
+		MenuObject* obj = m_menu->GetObject(i);
+		if (obj->IsButton() && obj != m_object) {
+			wxString id = obj->GetId();
+			if (m_object->GetFocusDest(navButton) == id)
+				sel = ctrl->GetCount();
+			long l = -1;
+			id.Mid(6).ToLong(&l);
+			wxString text = _("button") + wxString::Format(wxT(" %d"), (int) l);
+			if (obj->GetParam(wxT("text")).length())
+				text += wxT(": ") + obj->GetParam(_T("text"));
+			ctrl->Append(text, new wxString(id));
+		}
+	}
+	for (unsigned int i=0; i<m_menu->GetActionsCount(); i++) {
+		DVDAction* action = m_menu->GetAction(i);
+		wxString id = action->GetId();
+		if (m_object->GetFocusDest(navButton) == id)
+			sel = ctrl->GetCount();
+		ctrl->Append(id, new wxString(id));
+	}
+	ctrl->SetSelection(sel);
+	sizer->Add(grid, 0, 0, 0);
+}
+
+void MenuObjectPropDlg::CreateLook(wxBoxSizer* mainSizer) {
     // object look parameters
     bool lastChangeable = false;
+    wxFlexGridSizer* grid = NULL;
     for (int i=0; i<m_object->GetObjectParamsCount(); i++) {
 		MenuObjectParam* param = m_object->GetObjectParam(i);
 		if (param->name == wxT("rotation")) {
 			continue;
 		}
 
-		if (i == 0 || param->changeable != lastChangeable) {
+		if (grid == NULL || param->changeable != lastChangeable) {
 			if (param->changeable) {
-				grid = new wxFlexGridSizer(3, 4, 4, 16);
+				grid = new wxFlexGridSizer(4, 4, 16);
 				AddText(grid, wxT(""));
 				AddText(grid, _("Normal"));
 				AddText(grid, _("Highlighted"));
@@ -298,6 +373,33 @@ void MenuObjectPropDlg::CreateLook(wxBoxSizer* mainSizer, wxFlexGridSizer* grid)
 	}
 }
 
+void MenuObjectPropDlg::CreateAspectCtrls(wxSizer* sizer, wxSVG_PRESERVEASPECTRATIO align,
+		wxSVG_MEETORSLICE meetOrSlice) {
+	sizer->AddSpacer(4);
+	bool keepAspectRatio = align > wxSVG_PRESERVEASPECTRATIO_NONE;
+	BeginCheckGroup(sizer, _("Keep Aspect Ratio"), keepAspectRatio);
+	sizer->AddSpacer(4);
+	wxArrayString alignXStrings;
+	alignXStrings.Add(_("X Min"));
+	alignXStrings.Add(_("X Mid"));
+	alignXStrings.Add(_("X Max"));
+	int alignX = keepAspectRatio ? (align - wxSVG_PRESERVEASPECTRATIO_XMINYMIN) % 3 : 1;
+	sizer->Add(AddChoiceProp(alignXStrings[alignX], alignXStrings));
+	wxArrayString alignYStrings;
+	alignYStrings.Add(_("Y Min"));
+	alignYStrings.Add(_("Y Mid"));
+	alignYStrings.Add(_("Y Max"));
+	int alignY = keepAspectRatio ? (align - wxSVG_PRESERVEASPECTRATIO_XMINYMIN) / 3 : 1;
+	sizer->Add(AddChoiceProp(alignYStrings[alignY], alignYStrings), 0, wxLEFT, 4);
+	wxArrayString meetStrings;
+	meetStrings.Add(_("Meet"));
+	meetStrings.Add(_("Slice"));
+	int meet = align == wxSVG_PRESERVEASPECTRATIO_NONE || meetOrSlice == wxSVG_MEETORSLICE_SLICE ? 1 : 0;
+	sizer->Add(AddChoiceProp(meetStrings[meet], meetStrings), 0, wxLEFT, 4);
+	sizer->AddStretchSpacer(1);
+	EndGroup();
+}
+
 void MenuObjectPropDlg::CreateImageCrtls(wxFlexGridSizer* grid, const wxString& title, MenuObjectParam* param) {
 	wxString imageFile = m_object->GetParam(param->name);
 	wxString fname = imageFile.Find(wxT('#')) != -1 ? imageFile.BeforeLast(wxT('#')) : imageFile;
@@ -338,7 +440,7 @@ void MenuObjectPropDlg::CreateImageCrtls(wxFlexGridSizer* grid, const wxString&
 	if (!m_object->IsButton()) {
 		videoFrameSizer->Add(m_videoChoice, 0, wxALIGN_CENTER_VERTICAL);
 	}
-	m_videoFrameBt = new wxButton(this, VIDEOFRAME_BT_ID, wxT("..."));
+	m_videoFrameBt = new wxButton(propWindow, VIDEOFRAME_BT_ID, wxT("..."));
 	int h = m_videoFrameBt->GetSize().GetHeight() > 24 ? m_videoFrameBt->GetSize().GetHeight() : 24;
 	m_videoFrameBt->SetSizeHints(h, h, h, h);
 	videoFrameSizer->Add(m_videoFrameBt);
@@ -366,9 +468,12 @@ void MenuObjectPropDlg::CreateImageCrtls(wxFlexGridSizer* grid, const wxString&
 		m_videoPos = m_defaultPos;
 	// video duration
 	m_videoDuration = m_object->GetParamVideoDuration(param->name);
+	// keep aspect
+	wxSVGPreserveAspectRatio aspectRatio;
+	aspectRatio.SetValueAsString(m_object->GetParam(param->name, wxT("preserveAspectRatio")));
+	CreateAspectCtrls(videoFrameSizer, aspectRatio.GetAlign(), aspectRatio.GetMeetOrSlice());
 }
 
-
 void MenuObjectPropDlg::CreateShadowCtrls(wxFlexGridSizer* grid, const wxString& title, MenuObjectParam* param) {
 	bool shadow = m_object->GetParam(param->name, wxT("visibility")) != wxT("hidden");
 	BeginCheckGroup(grid, title, shadow, false);
@@ -388,7 +493,7 @@ void MenuObjectPropDlg::CreateShadowCtrls(wxFlexGridSizer* grid, const wxString&
 	AddSpinProp(shadowSizer, wxT(""), m_object->GetParamInt(param->name, wxT("x")), -999, 999, 46, wxT(""), false);
 	AddSpinProp(shadowSizer, wxT(""), m_object->GetParamInt(param->name, wxT("y")), -999, 999, 46, wxT(""), false);
 	shadowSizer->AddSpacer(4);
-	wxSVGElement* filter = m_object->GetElementById(param->element + wxT("Filter"));
+	wxSVGElement* filter = m_object->GetElementById(param->element.front() + wxT("Filter"));
 	if (filter && ((wxSVGElement*)filter->GetFirstChild())->GetDtd() == wxSVG_FEGAUSSIANBLUR_ELEMENT) {
 		wxSVGFEGaussianBlurElement* blur = (wxSVGFEGaussianBlurElement*) filter->GetFirstChild();
 		AddText(shadowSizer, _("Deviation:"));
@@ -436,6 +541,25 @@ void MenuObjectPropDlg::CreateLocAndSize(wxBoxSizer* mainSizer) {
 	EndGroup();
 }
 
+/**
+ * Creates panel with buttons
+ */
+void MenuObjectPropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton, bool dontShowCheckbox) {
+	wxStdDialogButtonSizer* buttonPane = new wxStdDialogButtonSizer();
+	
+	// cells & actions buttons
+	buttonPane->Add(new wxButton(this, ANIMATIONS_BT_ID, _("Animations...")));
+	buttonPane->Add(10, 10, 1, wxEXPAND);
+	
+	// ok & cancel buttons
+	buttonPane->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
+	buttonPane->AddButton(new wxButton(this, wxID_CANCEL, wxEmptyString));
+	
+	buttonPane->Realize();
+	buttonPane->GetAffirmativeButton()->SetDefault();
+	sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
+}
+
 int MenuObjectPropDlg::GetSelectedTsi() {
 	int tsi = 0;
 	if (m_targetChoice->GetSelection() >= 0)
@@ -718,49 +842,14 @@ void MenuObjectPropDlg::OnKeepAspectRatio(wxCommandEvent& evt) {
 	m_heightEdit->SetValue(m_widthEdit->GetValue());
 }
 
-void MenuObjectPropDlg::AddNavigationProp(wxSizer* sizer, NavigationButton navButton) {
-	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
-	AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString());
-	wxChoice* ctrl = (wxChoice*) m_controls[m_controls.Count()-1];
-	
-	// default navigation
-	wxString defBtId = m_object->GetDefaultFocusDest(navButton);
-	wxString text = _("none");
-	if (defBtId != m_object->GetId()) {
-		long l = -1;
-		if (defBtId.Mid(6).ToLong(&l))
-			text = _("button") + wxString::Format(wxT(" %d"), (int) l);
-	}
-	ctrl->Append(_("auto") + wxString(wxT(" (")) + text + wxT(")"), new wxString);
-	ctrl->Append(_("none"), new wxString(m_object->GetId()));
-
-	// add all buttons except current
-	int sel = m_object->GetFocusDest(navButton) == m_object->GetId() ? 1 : 0;
-	for (unsigned int i=0; i<m_menu->GetObjectsCount(); i++) {
-		MenuObject* obj = m_menu->GetObject(i);
-		if (obj->IsButton() && obj != m_object) {
-			wxString id = obj->GetId();
-			if (m_object->GetFocusDest(navButton) == id)
-				sel = ctrl->GetCount();
-			long l = -1;
-			id.Mid(6).ToLong(&l);
-			wxString text = _("button") + wxString::Format(wxT(" %d"), (int) l);
-			if (obj->GetParam(wxT("text")).length())
-				text += wxT(": ") + obj->GetParam(_T("text"));
-			ctrl->Append(text, new wxString(id));
-		}
-	}
-	for (unsigned int i=0; i<m_menu->GetActionsCount(); i++) {
-		DVDAction* action = m_menu->GetAction(i);
-		wxString id = action->GetId();
-		if (m_object->GetFocusDest(navButton) == id)
-			sel = ctrl->GetCount();
-		ctrl->Append(id, new wxString(id));
-	}
-	ctrl->SetSelection(sel);
-	sizer->Add(grid, 0, 0, 0);
+void MenuObjectPropDlg::OnAnimationsBt(wxCommandEvent& event) {
+	AnimationDlg dlg(this, m_object);
+	dlg.ShowModal();
 }
 
+/**
+ *  Populate values on the model
+ */
 bool MenuObjectPropDlg::SetValues() {
 	// check Action
 	int customRadioIdx = 5;
@@ -891,6 +980,19 @@ bool MenuObjectPropDlg::SetValues() {
 					m_object->SetParamImageVideo(param->name, imgFile, -1, -1);
 					m_object->SetDisplayVobId(DVD::MakeId(0, 0, 0));
 				}
+				// aspect ratio
+				if (GetBool(n++)) {
+					wxSVGPreserveAspectRatio aspectRatio;
+					int alignX = GetInt(n++);
+					int alignY = GetInt(n++);
+					aspectRatio.SetAlign((wxSVG_PRESERVEASPECTRATIO) (wxSVG_PRESERVEASPECTRATIO_XMINYMIN + alignY * 3 + alignX));
+					int meet = GetInt(n++);
+					aspectRatio.SetMeetOrSlice(meet == 0 ? wxSVG_MEETORSLICE_MEET : wxSVG_MEETORSLICE_SLICE);
+					m_object->SetParam(param->name, aspectRatio.GetValueAsString(), wxT("preserveAspectRatio"));
+				} else {
+					n += 3;
+					m_object->SetParam(param->name, wxT("none"), wxT("preserveAspectRatio"));
+				}
 			} else if (param->type == wxT("shadow")) {
 				m_object->SetParam(param->name, GetBool(n++) ? wxT("visible") : wxT("hidden"), wxT("visibility"));
 				m_object->SetParamColour(param->name, GetColour(n++));
@@ -898,7 +1000,7 @@ bool MenuObjectPropDlg::SetValues() {
 				m_object->SetParam(param->name, opacity, wxT("-opacity"));
 				m_object->SetParamInt(param->name, GetInt(n++), wxT("x"));
 				m_object->SetParamInt(param->name, GetInt(n++), wxT("y"));
-				wxSVGElement* filter = m_object->GetElementById(param->element + wxT("Filter"));
+				wxSVGElement* filter = m_object->GetElementById(param->element.front() + wxT("Filter"));
 				if (filter && ((wxSVGElement*)filter->GetFirstChild())->GetDtd() == wxSVG_FEGAUSSIANBLUR_ELEMENT) {
 					wxSVGFEGaussianBlurElement* blur = (wxSVGFEGaussianBlurElement*) filter->GetFirstChild();
 					double stdDeviation = GetDouble(n++);
