@@ -3,7 +3,7 @@
 // Purpose:     Settings dialog
 // Author:      Alex Thuering
 // Created:     27.03.2004
-// RCS-ID:      $Id: SettingsDlg.cpp,v 1.78 2015/01/31 13:34:46 ntalex Exp $
+// RCS-ID:      $Id: SettingsDlg.cpp,v 1.87 2016/10/05 19:51:30 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,6 +12,7 @@
 #include "Config.h"
 #include "Languages.h"
 #include "DVD.h"
+#include "Cache.h"
 #include <wxVillaLib/utils.h>
 #include <wx/notebook.h>
 #include <wx/filename.h>
@@ -21,15 +22,20 @@
 
 enum {
 	FILE_BROWSER_CHOICE_ID = 7900,
-	RESET_DONT_SHOW_FLAGS_BT_ID
+	RESET_DONT_SHOW_FLAGS_BT_ID,
+	CLEAR_CACHE_BT,
+	MODE_CHOICE_ID,
 };
 
 BEGIN_EVENT_TABLE(SettingsDlg, wxPropDlg)
 	EVT_CHOICE(FILE_BROWSER_CHOICE_ID, SettingsDlg::OnChangeFileBrowserDir)
 	EVT_BUTTON(RESET_DONT_SHOW_FLAGS_BT_ID, SettingsDlg::OnResetDontShowFlags)
+	EVT_BUTTON(CLEAR_CACHE_BT, SettingsDlg::OnClearCache)
+	EVT_CHOICE(MODE_CHOICE_ID, SettingsDlg::OnChangeEncoderMode)
 END_EVENT_TABLE()
 
-SettingsDlg::SettingsDlg(wxWindow* parent): wxPropDlg(parent), m_directoryEdit(NULL), m_directorySelectButton(NULL) {
+SettingsDlg::SettingsDlg(wxWindow* parent, Cache* cache): wxPropDlg(parent), m_cache(cache), m_directoryEdit(NULL),
+		m_directorySelectButton(NULL), encoderCtrl(NULL), modeCtrl(NULL), hqCtrl(NULL), xhqCtrl(NULL) {
 	Create(true);
 	SetTitle(_("Settings"));
 	SetSize(600, -1);
@@ -115,7 +121,10 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 	AddStretchSpacer(afSizer, 1);
 	
 	// default chapter length
-	AddSpinProp(interfaceGrid, _("Default chapter length:"), s_config.GetDefChapterLength(def), 0, 999, 60, _("min"));
+	wxSizer* chpSizer = AddSpinProp(interfaceGrid, _("Default chapter length:"), s_config.GetDefChapterLength(def),
+			0, 999, 60, _("min"), false);
+	AddSpacer(chpSizer, 8);
+	AddCheckProp(chpSizer, _("Add chapter at title end"), s_config.GetAddChapterAtTitleEnd(def));
 	
 	// default title post command
 	labels = DVD::GetDefPostCommandLabels();
@@ -192,8 +201,10 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 	options.Add(_("Yes"));
 	options.Add(_("No"));
 	AddRadioGroupProp(grpSizer, options, s_config.GetCacheClearPrompt(def));
-	AddStretchSpacer(grpSizer, 1);
 	EndGroup();
+	if (grpSizer)
+		grpSizer->Add(new wxButton(propWindow, CLEAR_CACHE_BT, _("Clear cache now")));
+	AddStretchSpacer(grpSizer, 1);
 	
 	// don't show fags
 	AddText(interfaceGrid, _("\"Don't show again\" flags:"));
@@ -223,32 +234,95 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 	}
 
 	// ----------------- Core Tab -------------------- //
-	wxSizer* menuSizer = AddSpinProp(coreGrid, _("Frame count of menu:"), s_config.GetMenuFrameCount(def), 1, 9999, 80,
+	wxSizer* lineSizer = AddSpinProp(coreGrid, _("Frame count of menu:"), s_config.GetMenuFrameCount(def), 1, 9999, 80,
 			wxT(""), false);
-	AddSpacer(menuSizer, 4);
-	AddCheckProp(menuSizer, _("Draw buttons on background"), s_config.GetDrawButtonsOnBackground(def));
-	AddSpinProp(coreGrid, _("Menu video bitrate:"), s_config.GetMenuVideoBitrate(def), 1, 9999, 80, _("KBit/s"));
-	AddSpinProp(coreGrid, _("Slideshow video bitrate:"), s_config.GetSlideshowVideoBitrate(def), 1, 9999, 80, _("KBit/s"));
+	AddSpacer(lineSizer, 4);
+	AddCheckProp(lineSizer, _("Draw buttons on background"), s_config.GetDrawButtonsOnBackground(def));
+	
+	lineSizer = AddSpinProp(coreGrid, _("Menu video bitrate:"), s_config.GetMenuVideoBitrate(def), 1, 9000, 80,
+			_("KBit/s"), false);
+	AddSpacer(lineSizer, 4);
+	AddCheckProp(lineSizer, wxT("CBR"), s_config.GetMenuVideoCBR(def));
+		
+	lineSizer = AddSpinProp(coreGrid, _("Slideshow video bitrate:"), s_config.GetSlideshowVideoBitrate(def), 1, 9000,
+			80, _("KBit/s"), false);
+	AddSpacer(lineSizer, 4);
+	AddCheckProp(lineSizer, wxT("CBR"), s_config.GetSlideshowVideoCBR(def));
+	
 	AddSpinProp(coreGrid, _("Default audio bitrate:"), s_config.GetAudioBitrate(def), 1, 9999, 80, _("KBit/s"));
 	AddSpinProp(coreGrid, _("Thread count:"), s_config.GetThreadCount(def), 1, 99, 80);
 	AddSpinProp(coreGrid, _("DVD reserved space:"), s_config.GetDvdReservedSpace(def), 0, 9999999, 80, _("KB"));
+	
+	// Encoder
+	wxArrayString encoders;
+	encoders.Add(_("internal"));
+#ifdef __WXMSW__
+	encoders.Add(wxT("ffmpeg-vbr"));
+#endif
+	int encoder = s_config.GetEncoder(def) == wxT("ffmpeg-vbr") ? 1 : 0;
+	wxSizer* encoderSizer = AddChoiceCustomProp(coreGrid, _("Encoder:"), encoders[encoder], encoders, 1, 0, false);
+	encoderCtrl = (wxChoice*) GetLastControl();
+	AddSpacer(encoderSizer, 4);
+	if (modes.size() == 0) {
+		modes.Add(wxT("cbr"));
+		modes.Add(wxT("vbr1pass"));
+		modes.Add(wxT("vbr2pass"));
+	}
+	wxArrayString modeLabels;
+	modeLabels.Add(_("auto"));
+	modeLabels.Add(_("CBR"));
+	modeLabels.Add(_("1-pass VBR"));
+	modeLabels.Add(_("2-pass VBR"));
+	int mode = modes.Index(s_config.GetEncoderMode(def).BeforeFirst(wxT('_')));
+	AddChoiceProp(encoderSizer, wxT(""), modeLabels[mode >= 0 ? mode + 1 : 0], modeLabels, -2, false, MODE_CHOICE_ID);
+	if (encoderSizer)
+		SetLastControlCustom(GetLastControlIndex() - 1, s_config.GetEncoder(def) == s_config.GetEncoder(true));
+	modeCtrl = (wxChoice*) GetLastControl();
+	AddSpacer(encoderSizer, 4);
+	AddCheckProp(encoderSizer, wxT("HQ"), s_config.GetEncoderMode(def).AfterFirst(wxT('_')) == wxT("hq"));
+	hqCtrl = (wxCheckBox*) GetLastControl();
+	AddSpacer(encoderSizer, 4);
+	AddCheckProp(encoderSizer, wxT("XHQ"), s_config.GetEncoderMode(def).AfterFirst(wxT('_')) == wxT("xhq"));
+	xhqCtrl = (wxCheckBox*) GetLastControl();
+	wxCommandEvent evt;
+	OnChangeEncoderMode(evt);
+	
 	AddTextProp(coreGrid, _("Create ISO command:"), s_config.GetIsoCmd(def));
 	AddTextProp(coreGrid, _("Burn command:"), s_config.GetBurnCmd(def));
 	AddTextProp(coreGrid, _("Burn ISO command:"), s_config.GetBurnISOCmd(def));
 	AddTextProp(coreGrid, _("Add ECC command:"), s_config.GetAddECCCmd(def));
 	AddTextProp(coreGrid, _("Format command:"), s_config.GetFormatCmd(def));
-	labels.clear();;
+	
+	AddText(coreGrid, _("Use mplex:"));
+	grpSizer = BeginGroup(coreGrid, wxT(""));
+	labels.clear();
 	labels.Add(_("Yes"));
 	labels.Add(_("No"));
 	labels.Add(_("For menus only"));
 	int opt = s_config.GetUseMplex(def) ? 0 : (s_config.GetUseMplexForMenus(def) ? 2 : 1);
-	AddChoiceProp(coreGrid, _("Use mplex:"), labels[opt], labels, 0, false);
+	AddRadioGroupProp(grpSizer, labels, opt);
+	EndGroup();
+	
+	AddText(coreGrid, _("NTSC film:"));
+	AddCheckProp(coreGrid, _("reencode by default"), s_config.GetDefRencodeNtscFilm(def));
 
 	grpSizer = BeginGroup(debugSizer, _("Debug"));
 	AddCheckProp(grpSizer, _("Don't remove temp files"), !s_config.GetRemoveTempFiles(def));
 }
 
 bool SettingsDlg::SetValues() {
+	// check encoder
+#ifdef __WXMSW__
+	if (encoderCtrl->GetSelection() > 0) {
+		if (!wxFileExists(wxGetAppPath() + encoderCtrl->GetStringSelection() + wxT(".exe"))
+				&& !wxFileExists(wxGetAppPath() + encoderCtrl->GetStringSelection() + wxT(".bat"))) {
+			wxMessageBox(wxT("Encoder ") + encoderCtrl->GetStringSelection() + wxT(" is not found. Please install."),
+					GetTitle(), wxOK|wxICON_ERROR, this);
+			return false;
+		}
+	}
+#endif
+	
 	// interface settings
 	int i = 0;
 	if (GetLangCode(GetInt(i++)) != s_config.GetLanguageCode()) {
@@ -269,6 +343,7 @@ bool SettingsDlg::SetValues() {
 	s_config.SetDefAudio51(GetBool(i++));
 	s_config.SetDefAudioNormalize(GetBool(i++));
 	s_config.SetDefChapterLength(GetInt(i++));
+	s_config.SetAddChapterAtTitleEnd(GetBool(i++));
 	s_config.SetDefPostCommand(GetInt(i++));
 	s_config.SetDefButton(m_buttons[GetInt(i++)]);
 	s_config.SetAcceptInvalidActions(GetBool(i++));
@@ -288,10 +363,21 @@ bool SettingsDlg::SetValues() {
 	s_config.SetMenuFrameCount(GetInt(i++));
 	s_config.SetDrawButtonsOnBackground(GetBool(i++));
 	s_config.SetMenuVideoBitrate(GetInt(i++));
+	s_config.SetMenuVideoCBR(GetBool(i++));
 	s_config.SetSlideshowVideoBitrate(GetInt(i++));
+	s_config.SetSlideshowVideoCBR(GetBool(i++));
 	s_config.SetAudioBitrate(GetInt(i++));
 	s_config.SetThreadCount(GetInt(i++));
 	s_config.SetDvdReservedSpace(GetInt(i++));
+	wxString encoder = GetInt(i++) > 0 ? GetString(i - 1) : wxT("");
+	s_config.SetEncoder(encoder);
+	wxString mode = encoder.length() == 0 || GetInt(i) == 0 ? wxT("") : modes[GetInt(i) - 1];
+	if (GetBool(i+1) && mode.length())
+		mode += wxT("_hq");
+	else if (GetBool(i+2) && mode.length())
+		mode += wxT("_xhq");
+	s_config.SetEncoderMode(mode);
+	i += 3;
 	s_config.SetIsoCmd(GetString(i++));
 	s_config.SetBurnCmd(GetString(i++));
 	s_config.SetBurnISOCmd(GetString(i++));
@@ -300,6 +386,7 @@ bool SettingsDlg::SetValues() {
 	int mplex = GetInt(i++);
 	s_config.SetUseMplex(mplex == 0);
 	s_config.SetUseMplexForMenus(mplex == 0 || mplex == 2);
+	s_config.SetDefRencodeNtscFilm(GetBool(i++));
 	s_config.SetRemoveTempFiles(!GetBool(i++));
 	return true;
 }
@@ -320,6 +407,20 @@ void SettingsDlg::OnResetDontShowFlags(wxCommandEvent& evt) {
 			wxOK | wxCENTRE | wxICON_INFORMATION);
 }
 
+void SettingsDlg::OnClearCache(wxCommandEvent& evt) {
+	if (m_cache->Clear())
+		wxMessageBox(_("The transcoding cache is successfully cleared."), _("Settings"),
+				wxOK | wxCENTRE | wxICON_INFORMATION);
+}
+
+void SettingsDlg::OnChangeEncoderMode(wxCommandEvent& evt) {
+	hqCtrl->Enable(modeCtrl->GetSelection() > 0);
+	if (!hqCtrl->IsEnabled())
+		hqCtrl->SetValue(false);
+	xhqCtrl->Enable(modeCtrl->GetSelection() == 3);
+	if (!xhqCtrl->IsEnabled())
+		xhqCtrl->SetValue(false);
+}
 
 void SettingsDlg::AddSpacer(wxSizer* sizer, int size) {
 	if (sizer)
