@@ -3,14 +3,14 @@
 // Purpose:     The subtitle properties dialog
 // Author:      Alex Thuering
 // Created:     24.02.2010
-// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.12 2013/04/14 05:29:05 ntalex Exp $
+// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.15 2016/04/19 23:09:52 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "SubtitlePropDlg.h"
 #include "DVD.h"
-#include "Utils.h"
+#include "Config.h"
 #include <wx/filename.h>
 #include <wx/colordlg.h>
 #include <wxVillaLib/utils.h>
@@ -22,6 +22,7 @@
 
 //(*IdInit(SubtitlePropDlg)
 const long SubtitlePropDlg::ID_CHOICE1 = wxNewId();
+const long SubtitlePropDlg::ID_FORCE_CB = wxNewId();
 const long SubtitlePropDlg::ID_CHOICE2 = wxNewId();
 const long SubtitlePropDlg::FONT_FAMILY_BOX_ID = wxNewId();
 const long SubtitlePropDlg::FONT_STYLE_BOX_ID = wxNewId();
@@ -54,6 +55,8 @@ const long SubtitlePropDlg::ID_SPINCTRL3 = wxNewId();
 const long SubtitlePropDlg::ID_SPINCTRL4 = wxNewId();
 const long SubtitlePropDlg::ID_SPINCTRL5 = wxNewId();
 const long SubtitlePropDlg::ID_SPINCTRL6 = wxNewId();
+const long SubtitlePropDlg::ID_DEF_BT = wxNewId();
+const long SubtitlePropDlg::ID_BUTTON1 = wxNewId();
 //*)
 
 BEGIN_EVENT_TABLE(SubtitlePropDlg,wxDialog)
@@ -64,10 +67,11 @@ END_EVENT_TABLE()
 SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxString& langCode) {
 	m_textsub = textsub;
 	m_langCode = langCode;
-		
+
 	//(*Initialize(SubtitlePropDlg)
 	wxStaticText* staticText2;
 	wxGridSizer* alignmentSizer;
+	wxButton* resetBt;
 	wxStaticText* staticText5;
 	wxBoxSizer* boxSizer1;
 	wxStaticText* staticText4;
@@ -84,10 +88,12 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	wxStaticText* staticText1;
 	wxBoxSizer* fillSizer;
 	wxBoxSizer* propSizer2;
+	wxBoxSizer* BoxSizer1;
 	wxStaticText* staticText11;
 	wxBoxSizer* fSizer1;
 	wxBoxSizer* shadowSizer;
 	wxStaticText* staticText6;
+	wxBoxSizer* btSizer;
 	wxBoxSizer* mainSizer;
 	wxGridSizer* marginSizer;
 	wxBoxSizer* fSizer3;
@@ -100,8 +106,13 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	propSizer->AddGrowableRow(2);
 	languageLabel = new wxStaticText(this, wxID_ANY, _("Language:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propSizer->Add(languageLabel, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
+	BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
 	m_langChoice = new wxChoice(this, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, 0, 0, 0, wxDefaultValidator, _T("ID_CHOICE1"));
-	propSizer->Add(m_langChoice, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
+	BoxSizer1->Add(m_langChoice, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
+	m_force = new wxCheckBox(this, ID_FORCE_CB, _("Force display"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_FORCE_CB"));
+	m_force->SetValue(false);
+	BoxSizer1->Add(m_force, 0, wxLEFT|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	propSizer->Add(BoxSizer1, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
 	staticText1 = new wxStaticText(this, wxID_ANY, _("Charset:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propSizer->Add(staticText1, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
 	m_charsetChoice = new wxChoice(this, ID_CHOICE2, wxDefaultPosition, wxDefaultSize, 0, 0, 0, wxDefaultValidator, _T("ID_CHOICE2"));
@@ -227,11 +238,17 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	propSizer2->Add(staticBoxSizer2, 0, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP, 0);
 	propSizer->Add(propSizer2, 1, wxTOP|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 6);
 	mainSizer->Add(propSizer, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 10);
+	btSizer = new wxBoxSizer(wxHORIZONTAL);
+	m_useAsDefaultBt = new wxButton(this, ID_DEF_BT, _("&Use as defaults"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_DEF_BT"));
+	btSizer->Add(m_useAsDefaultBt, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	resetBt = new wxButton(this, ID_BUTTON1, _("Reset"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
+	btSizer->Add(resetBt, 0, wxLEFT|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
 	stdDialogButtonSizer = new wxStdDialogButtonSizer();
 	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
 	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_CANCEL, wxEmptyString));
 	stdDialogButtonSizer->Realize();
-	mainSizer->Add(stdDialogButtonSizer, 0, wxBOTTOM|wxLEFT|wxRIGHT|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 4);
+	btSizer->Add(stdDialogButtonSizer, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 4);
+	mainSizer->Add(btSizer, 0, wxBOTTOM|wxLEFT|wxRIGHT|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 4);
 	SetSizer(mainSizer);
 	mainSizer->Fit(this);
 	mainSizer->SetSizeHints(this);
@@ -244,9 +261,11 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	Connect(FILL_BT_ID,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SubtitlePropDlg::OnSelectFillColor);
 	Connect(OUTLINE_BT_ID,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SubtitlePropDlg::OnSelectOutlineColor);
 	Connect(SHADOW_BT_ID,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SubtitlePropDlg::OnSelectShadowColor);
+	Connect(ID_DEF_BT,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SubtitlePropDlg::OnUseAsDefaults);
+	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SubtitlePropDlg::OnResetBt);
 	//*)
 	stdDialogButtonSizer->GetAffirmativeButton()->SetDefault();
-    
+
     m_langChoice->Append(DVD::GetLanguageNames());
     map<wxString, wxString>& langMap = DVD::GetLanguageMap();
 	for (map<wxString, wxString>::iterator langIt = langMap.begin(); langIt != langMap.end(); langIt++) {
@@ -254,6 +273,8 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 			m_langChoice->SetStringSelection(langIt->first);
 		}
 	}
+	
+	m_force->SetValue(m_textsub->GetForce());
 
 	m_charsetChoice->Append(DVD::GetCharsets());
 	m_charsetChoice->SetStringSelection(m_textsub->GetCharacterSet());
@@ -266,7 +287,7 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	wxCommandEvent evt;
 	OnFontFamilySelected(evt);
 	m_fontStyleBox->SetStringSelection(m_textsub->GetFontStyle());
-	
+
 	for (int i=6; i<72; i++) {
 		if (i>18) {
 			i++;
@@ -277,11 +298,11 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 		}
 		m_fontSizeBox->Append(wxString::Format(wxT("%d"), i));
 	}
-	
+
 	m_fontSizeSpin->SetValue((int)m_textsub->GetFontSize());
 	wxSpinEvent spnEvt;
 	OnFontSizeChanged(spnEvt);
-	
+
 	// colours
 	SetFillColour(m_textsub->GetFillColour());
 	SetOutlineColour(m_textsub->GetOutlineColour());
@@ -289,7 +310,7 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	m_thicknessCtrl->SetValue(wxString::Format(wxT("%0.1f"), m_textsub->GetOutlineThickness()));
 	m_offsetXCtrl->SetValue(m_textsub->GetShadowOffset().x);
 	m_offsetYCtrl->SetValue(m_textsub->GetShadowOffset().y);
-	
+
 	// align
 	m_alignRadioTL->SetValue(m_textsub->GetAlignment() == (wxALIGN_TOP | wxALIGN_LEFT));
 	m_alignRadioTC->SetValue(m_textsub->GetAlignment() == (wxALIGN_TOP | wxALIGN_CENTER_HORIZONTAL));
@@ -303,12 +324,12 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent,TextSub* textsub, const wxStri
 	m_marginRight->SetValue(m_textsub->GetRightMargin());
 	m_marginTop->SetValue(m_textsub->GetTopMargin());
 	m_marginBottom->SetValue(m_textsub->GetBottomMargin());
-	
+
     m_langChoice->SetMinSize(wxSize(m_fontFamilyBox->GetSize().GetWidth(), -1));
     m_charsetChoice->SetMinSize(wxSize(m_fontFamilyBox->GetSize().GetWidth(), -1));
     OnFontChanged();
     m_langChoice->SetFocus();
-    
+
 	mainSizer->Fit(this);
 	mainSizer->SetSizeHints(this);
     SetMinSize(GetSize());
@@ -389,11 +410,12 @@ void SubtitlePropDlg::OnFontChanged() {
 int SubtitlePropDlg::ShowModal() {
 	int res = wxDialog::ShowModal();
 	if (res == wxID_OK) {
+		m_textsub->SetForce(m_force->IsChecked());
 		m_textsub->SetCharacterSet(m_charsetChoice->GetStringSelection());
 		m_textsub->SetFontFamily(m_fontFamilyBox->GetStringSelection());
 		m_textsub->SetFontStyle(m_fontStyleBox->GetStringSelection());
 		m_textsub->SetFontSize(m_fontSizeSpin->GetValue());
-		
+
 		// colours
 		m_textsub->SetFillColour(GetFillColour());
 		m_textsub->SetOutlineColour(GetOutlineColour());
@@ -402,7 +424,7 @@ int SubtitlePropDlg::ShowModal() {
 		if (m_thicknessCtrl->GetValue().ToDouble(&dval))
 			m_textsub->SetOutlineThickness(dval);
 		m_textsub->SetShadowOffset(wxPoint(m_offsetXCtrl->GetValue(), m_offsetYCtrl->GetValue()));
-		
+
 		// alignment
 		if (m_alignRadioTL->GetValue())
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_TOP | wxALIGN_LEFT));
@@ -420,7 +442,7 @@ int SubtitlePropDlg::ShowModal() {
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL));
 		else if (m_alignRadioBR->GetValue())
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_RIGHT));
-		
+
 		// margin
 		m_textsub->SetLeftMargin(m_marginLeft->GetValue());
 		m_textsub->SetRightMargin(m_marginRight->GetValue());
@@ -503,3 +525,76 @@ void SubtitlePropDlg::SetShadowColour(const wxColour& colour) {
 	if (colour.IsOk())
 		m_shadowOpacityCtrl->SetValue(lround(((double) colour.Alpha()) * 100 / 255));
 }
+
+void SubtitlePropDlg::OnUseAsDefaults(wxCommandEvent& event) {
+	s_config.SetSubtitlesCharacterSet(m_charsetChoice->GetStringSelection());
+	s_config.SetSubtitlesFontFamily(m_fontFamilyBox->GetStringSelection());
+	s_config.SetSubtitlesFontStyle(m_fontStyleBox->GetStringSelection());
+	s_config.SetSubtitlesFontSize(m_fontSizeSpin->GetValue());
+
+	// colours
+	s_config.SetSubtitlesFillColour(GetFillColour());
+	s_config.SetSubtitlesOutlineColour(GetOutlineColour());
+	s_config.SetSubtitlesShadowColour(GetShadowColour());
+	double dval;
+	if (m_thicknessCtrl->GetValue().ToDouble(&dval))
+		s_config.SetSubtitlesOutlineThickness(dval);
+	s_config.SetSubtitlesShadowOffsetX(m_offsetXCtrl->GetValue());
+	s_config.SetSubtitlesShadowOffsetY(m_offsetYCtrl->GetValue());
+
+	// alignment
+	if (m_alignRadioTL->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_TOP | wxALIGN_LEFT));
+	else if (m_alignRadioTC->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_TOP | wxALIGN_CENTER_HORIZONTAL));
+	else if (m_alignRadioTR->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_TOP | wxALIGN_RIGHT));
+	else if (m_alignRadioCL->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT));
+	else if (m_alignRadioCR->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT));
+	else if (m_alignRadioBL->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_LEFT));
+	else if (m_alignRadioBC->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL));
+	else if (m_alignRadioBR->GetValue())
+		s_config.SetSubtitlesAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_RIGHT));
+
+	// margin
+	s_config.SetSubtitlesLeftMargin(m_marginLeft->GetValue());
+	s_config.SetSubtitlesRightMargin(m_marginRight->GetValue());
+	s_config.SetSubtitlesTopMargin(m_marginTop->GetValue());
+	s_config.SetSubtitlesBottomMargin(m_marginBottom->GetValue());
+	s_config.Flush();
+}
+
+void SubtitlePropDlg::OnResetBt(wxCommandEvent& event) {
+	m_charsetChoice->SetStringSelection(s_config.GetSubtitlesCharacterSet(true));
+	m_fontFamilyBox->SetStringSelection(s_config.GetSubtitlesFontFamily(true));
+	m_fontStyleBox->SetStringSelection(s_config.GetSubtitlesFontStyle(true));
+	m_fontSizeSpin->SetValue(s_config.GetSubtitlesFontSize(true));
+
+	// colours
+	SetFillColour(s_config.GetSubtitlesFillColour(true));
+	SetOutlineColour(s_config.GetSubtitlesOutlineColour(true));
+	SetShadowColour(s_config.GetSubtitlesShadowColour(true));
+	m_thicknessCtrl->SetValue(wxString::Format(wxT("%0.1f"), s_config.GetSubtitlesOutlineThickness(true)));
+	m_offsetXCtrl->SetValue(s_config.GetSubtitlesShadowOffsetX(true));
+	m_offsetYCtrl->SetValue(s_config.GetSubtitlesShadowOffsetY(true));
+
+	// alignment
+	m_alignRadioTL->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_TOP | wxALIGN_LEFT));
+	m_alignRadioTC->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_TOP | wxALIGN_CENTER_HORIZONTAL));
+	m_alignRadioTR->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_TOP | wxALIGN_RIGHT));
+	m_alignRadioCL->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT));
+	m_alignRadioCR->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT));
+	m_alignRadioBL->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_BOTTOM | wxALIGN_LEFT));
+	m_alignRadioBC->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL));
+	m_alignRadioBR->SetValue(s_config.GetSubtitlesAlignment(true) == (wxALIGN_BOTTOM | wxALIGN_RIGHT));
+	
+	// margin
+	m_marginLeft->SetValue(s_config.GetSubtitlesLeftMargin(true));
+	m_marginRight->SetValue(s_config.GetSubtitlesRightMargin(true));
+	m_marginTop->SetValue(s_config.GetSubtitlesTopMargin(true));
+	m_marginBottom->SetValue(s_config.GetSubtitlesBottomMargin(true));
+}
