@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.63 2013/12/29 21:41:13 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.65 2016/05/11 20:02:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -101,8 +101,16 @@ void TitlePropDlg::CreateVobGroup(wxSizer* sizer) {
 	wxBoxSizer* vobPane = new wxBoxSizer(wxHORIZONTAL);
 	grpSizer->Add(vobPane, 1, wxEXPAND);
 	
-	m_vobListBox = new VobListBox(this, VOB_BOX_ID, m_vob, m_pgcs, m_dvd);
-	vobPane->Add(m_vobListBox, 1, wxEXPAND);
+	wxScrolledWindow* scrollWin = new wxScrolledWindow(this);
+	m_vobListBox = new VobListBox(scrollWin, VOB_BOX_ID, m_vob, m_pgcs, m_dvd, this);
+	m_vobListBox->Fit();
+	wxBoxSizer* scrollSize = new wxBoxSizer(wxVERTICAL);
+	scrollSize->Add(m_vobListBox, 1, wxEXPAND);
+	scrollWin->SetSizer(scrollSize);
+	scrollWin->SetMinSize(wxSize(m_vobListBox->GetClientSize().GetWidth() + 32, 300));
+	scrollWin->FitInside();
+	scrollWin->SetScrollRate(5, 5);
+	vobPane->Add(scrollWin, 1, wxEXPAND);
 	
 	wxBoxSizer* buttonPane = new wxBoxSizer(wxVERTICAL);
 	vobPane->Add(buttonPane, 0, wxEXPAND|wxLEFT, 4);
@@ -494,23 +502,29 @@ void TitlePropDlg::UpdateDoNotTranscodeCheck() {
 	m_doNotTranscodeCheckBox->SetValue(m_vobListBox->GetDoNotTranscode());
 	m_doNotTranscodeCheckBox->Enable(!m_vobListBox->HasAudioFiles() && MPEG::HasNavPacks(m_vob->GetFilename()));
 }
-
 void TitlePropDlg::OnChaptersBt(wxCommandEvent& event) {
+	if (!ApplyChaptersCtrl())
+		return;
+	ChaptersDlg dialog(this, m_dvd, m_tsi, m_pgci, m_vobi, m_vob);
+	if (dialog.ShowModal() == wxID_OK)
+		UpdateChaptersCtrl();
+}
+
+bool TitlePropDlg::ApplyChaptersCtrl() {
 	// update
 	if (m_chaptersCtrl->IsEnabled()) {
 		wxStringTokenizer tkz(m_chaptersCtrl->GetValue(), _T(","));
 		while (tkz.HasMoreTokens()) {
 			wxString token = tkz.GetNextToken().Strip(wxString::both);
 			if (!s_chapterRegEx.Matches(token)) {
-				wxLogError(_("Invalid chapter timestamp '%s'"), token.c_str());
-				return;
+				wxLogError
+				(_("Invalid chapter timestamp '%s'"), token.c_str());
+				return false;
 			}
 		}
 		m_vob->SetChapters(m_chaptersCtrl->GetValue(), m_vobi == 0);
 	}
-	ChaptersDlg dialog(this, m_dvd, m_tsi, m_pgci, m_vobi, m_vob);
-	if (dialog.ShowModal() == wxID_OK)
-		UpdateChaptersCtrl();
+	return true;
 }
 
 void TitlePropDlg::UpdateChaptersCtrl() {
