@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Titleset
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: Titleset.cpp,v 1.14 2014/01/19 17:11:07 ntalex Exp $
+// RCS-ID:      $Id: Titleset.cpp,v 1.16 2016/03/12 17:48:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,10 +11,12 @@
 #include "Titleset.h"
 #include "DVD.h"
 #include "Menu.h"
+#include "Utils.h"
 #include "Config.h"
-#include <wx/sstream.h>
-#include <wxSVGXML/svgxmlhelpr.h>
 #include <wxVillaLib/utils.h>
+#include <wxSVGXML/svgxmlhelpr.h>
+#include <wx/sstream.h>
+#include <wx/log.h>
 
 ////////////////////////////// Video ///////////////////////////////////
 wxArrayString Video::GetAspectStrings(wxString def) {
@@ -106,6 +108,10 @@ bool SubStream::PutXML(wxSvgXmlNode* node) {
 }
 
 ////////////////////////////////// SubPicture //////////////////////////////////////
+/** Destructor */
+SubPicture::~SubPicture() {
+	VECTOR_CLEAR(m_streams, SubStream)
+}
 
 wxSvgXmlNode* SubPicture::GetXML(DVDFileType type, Video& video, bool menu) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("subpicture"));
@@ -147,6 +153,10 @@ bool SubPicture::PutXML(wxSvgXmlNode* node) {
 }
 
 ////////////////////////////////// PgcArray //////////////////////////////////////
+/** Destructor */
+PgcArray::~PgcArray() {
+	VECTOR_CLEAR(m_subPictures, SubPicture)
+}
 
 unsigned int PgcArray::GetAudioStreamCount() {
 	unsigned int count = 1; // at least one audio stream
