@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.100 2015/01/31 16:10:01 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.108 2016/12/17 17:27:38 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -202,8 +202,10 @@ void TitlesetManager::SetDVD(DVD* dvd) {
 wxBitmap TitlesetManager::GetThumbImage(wxThumb& thumb) {
 	if (!m_dvd)
 		return wxBitmap();
+#ifndef __WXGTK__
 	if (m_progressDlg)
 		m_progressDlg->Pulse();
+#endif
 	int id = thumb.GetId();
 	Vob* vob = m_dvd->GetVob(DVD::GetTsi(id), DVD::IsMenu(id), DVD::GetPgci(id), DVD::GetVobi(id));
 	wxBitmap img = thumb.GetBitmap();
@@ -425,9 +427,9 @@ void TitlesetManager::UpdateItems(wxProgressDialog* progressDlg) {
 	SetThumbSize(96, 76 + (m_dvd->GetTitlesets().size() > 1 || m_dvd->GetVmgm().size() > 0 ? m_tTextHeight : 0));
 	UpdateProp();
 	Refresh();
-	wxSplitterWindow* p = (wxSplitterWindow*) GetParent(); 
+	wxSplitterWindow* p = (wxSplitterWindow*) GetParent();
 	p->SetSashPosition(p->GetSashPosition() + (m_rows > 1 ? 0 : 1000));
-	m_progressDlg = NULL;;
+	m_progressDlg = NULL;
 }
 
 int TitlesetManager::GetIndex(int tsi, int pgci, int vobi, bool menu) {
@@ -442,6 +444,46 @@ int TitlesetManager::GetIndex(int id) {
 	return -1;
 }
 
+/** Checks if video fit on DVD without re-encoding */
+bool TitlesetManager::AskForReencoding(VideoFormat videoFormat) {
+	// check if "copy" is selected for video
+	bool isCopy = false;
+	for (int tsi = 0; tsi < (int) (m_dvd->GetTitlesets().Count()); tsi++) {
+		Titleset* ts = m_dvd->GetTitlesets()[tsi];
+		for (int pgci = 0; pgci < (int) (ts->GetTitles().Count()); pgci++) {
+			Pgc* pgc = ts->GetTitles()[pgci];
+			for (int vobi = 0; vobi < (int) (pgc->GetVobs().Count()); vobi++) {
+				Stream* stream = pgc->GetVobs()[vobi]->GetVideoStream();
+				if (stream != NULL && stream->GetVideoFormat() == vfCOPY) {
+					isCopy = true;
+					break;
+				}
+			}
+		}
+	}
+	// Ask to enable re-encoding
+	if (isCopy && m_dvd->GetCapacity() != dcUNLIMITED && m_dvd->GetSize() > m_dvd->GetCapacityValue()) {
+		if (wxMessageBox(_("The selected video files don't fit on DVD without re-encoding.\n\
+Do you want to enable re-encoding?"), _("Burn"), wxYES_NO | wxICON_QUESTION, this) == wxYES) {
+			m_reEncodingByDefault = true;
+			for (int tsi = 0; tsi < (int) (m_dvd->GetTitlesets().Count()); tsi++) {
+				Titleset* ts = m_dvd->GetTitlesets()[tsi];
+				for (int pgci = 0; pgci < (int) (ts->GetTitles().Count()); pgci++) {
+					Pgc* pgc = ts->GetTitles()[pgci];
+					for (int vobi = 0; vobi < (int) (pgc->GetVobs().Count()); vobi++) {
+						Stream* stream = pgc->GetVobs()[vobi]->GetVideoStream();
+						if (stream != NULL)
+							stream->SetDestinationFormat(videoFormat);
+					}
+				}
+			}
+			SendDvdChangedEvent();
+		}
+		return true;
+	}
+	return false;
+}
+
 bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi, bool allowMenuDuplication) {
 	wxProgressDialog pdlg(wxT("DVDStyler"), wxString::Format(_("Adding file %s.\nPlease wait..."),
 			wxFileNameFromPath(fname).c_str()), 99, this->GetParent(), wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH);
@@ -464,25 +506,33 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 		delete vob;
 		return AddAudio(fname);
 	}
-
+	
 	// set chapters
-	int chapterCount = 0;
-	if (s_config.GetDefChapterLength() > 0) {
-		if (vob->GetDuration() > 0) {
-			int duration = (int) vob->GetDuration() / 60;
-			chapterCount = duration / s_config.GetDefChapterLength();
-			if (chapterCount > 0 && duration - s_config.GetDefChapterLength()*chapterCount < s_config.GetDefChapterLength()/5)
-				chapterCount--;
-			if (chapterCount>100)
-				chapterCount = 100;
-		} else {
-			chapterCount = 10;
+	int chapterCount = vob->GetChapterCount();
+	if (chapterCount == 0) {
+		if (s_config.GetDefChapterLength() > 0) {
+			if (vob->GetDuration() > 0) {
+				int duration = (int) vob->GetDuration() / 60;
+				chapterCount = duration / s_config.GetDefChapterLength();
+				if (chapterCount > 0
+						&& duration - s_config.GetDefChapterLength()*chapterCount < s_config.GetDefChapterLength()/5)
+					chapterCount--;
+				if (chapterCount > 100)
+					chapterCount = 100;
+			} else {
+				chapterCount = 10;
+			}
+		}
+		long t = 0;
+		for (int i = 0; i <= chapterCount; i++) {
+			vob->GetCells().push_back(new Cell(t * 60000));
+			t += s_config.GetDefChapterLength();
 		}
 	}
-	long t = 0;
-	for (int i = 0; i <= chapterCount; i++) {
-		vob->GetCells().push_back(new Cell(t * 60000));
-		t += s_config.GetDefChapterLength();
+	// add a chapter near the end - fix for the "next" button and post command for some players
+	if (s_config.GetAddChapterAtTitleEnd() && vob->GetDuration() > 1) {
+		int duration = (int) vob->GetDuration();
+		vob->GetCells().push_back(new Cell((duration - 1) * 1000));
 	}
 		
 	int videoBitrateOld = m_dvd->GetVideoBitrate();
@@ -521,11 +571,11 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 			if (isNTSC(m_dvd->GetVideoFormat()) && fps == 25) {
 				if (wxMessageBox(_("This file contains video in PAL format. Do you want to change DVD format to PAL too?"),
 						wxT("DVDStyler"), wxYES_NO|wxICON_QUESTION, this) == wxYES)
-					m_dvd->SetVideoFormat(vfPAL, true);
+					m_dvd->SetVideoFormat(vfPAL);
 			} else if (!isNTSC(m_dvd->GetVideoFormat()) && (fps == 24 || fps == 30)) {
 				if (wxMessageBox(_("This file contains video in NTSC format. Do you want to change DVD format to NTSC too?"),
 						wxT("DVDStyler"), wxYES_NO|wxICON_QUESTION, this) == wxYES)
-					m_dvd->SetVideoFormat(vfNTSC, true);
+					m_dvd->SetVideoFormat(vfNTSC);
 			}
 		}
 	} else if (createTitle && pgcs.Count() >= 99) {
@@ -535,22 +585,23 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 	if (pgcs.GetVideo().GetFormat() <= vfCOPY)
 		pgcs.GetVideo().SetFormat(m_dvd->GetVideoFormat());
 	// check if reencoding is needed
-	int audioStreamIdx = 0;
+	unsigned int audioStreamIdx = 0;
+	unsigned int subtitleStreamIdx = 0;
 	for (unsigned int i=0; i<vob->GetStreams().size(); i++) {
 		Stream* stream = vob->GetStreams()[i];
 		if (stream->GetType() == stVIDEO) {
 			// set destination format (default COPY) if codec is not mpeg2
 			if (m_reEncodingByDefault
 					|| vob->GetStreams().size() == 1 // only video is available
-					|| !stream->GetSourceCodecName().StartsWith(wxT("mpeg2video"))
+					|| !stream->IsDvdCompliant()
 					|| stream->GetSourceVideoFormat() != pgcs.GetVideo().GetFormat()
 					|| (s_config.GetDefRencodeNtscFilm() && lround(stream->GetSourceFps()) == 24)
-					|| fabs(stream->GetSourceAspectRatio()
-							- GetFrameAspectRatio(vfPAL, pgcs.GetVideo().GetAspect())) >= 0.15) {
+					|| fabs(stream->GetSourceAspectRatio() - GetFrameAspectRatio(pgcs.GetVideo().GetAspect())) >= 0.1) {
 				stream->SetDestinationFormat(pgcs.GetVideo().GetFormat());
 				vob->SetKeepAspectRatio(s_config.GetDefKeepAspectRatio());
-			} else if (vob->GetAudioFilenames().Count() == 0 && MPEG::HasNavPacks(vob->GetFilename()) && i == 0)
+			} else if (vob->GetAudioFilenames().Count() == 0 && MPEG::HasNavPacks(vob->GetFilename())) {
 				vob->SetDoNotTranscode(true);  // do not remultiplex/transcode if the video is already a VOB
+			}
 		} else if (stream->GetType() == stAUDIO) {
 			// set destination format (default COPY) if codec is not mp2/ac3 or sample rate is not 48 kHz
 			if (stream->GetSourceAudioFormat() != m_dvd->GetAudioFormat()
@@ -558,8 +609,8 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 				stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 				vob->SetDoNotTranscode(false);
 			}
-			if (s_config.GetDefAudio51() && m_dvd->GetAudioFormat() == afAC3
-					&& stream->GetSourceChannelNumber() != 6) {
+			if (s_config.GetDefAudio51() && m_dvd->GetAudioFormat() >= afAC3
+					&& stream->GetSourceChannelNumber() < 6) {
 				stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 				stream->SetChannelNumber(6);
 				vob->SetDoNotTranscode(false);
@@ -569,9 +620,26 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 				vob->SetDoNotTranscode(false);
 				stream->SetAudioAdjustType(aatNormalize);
 			}
+			// set language
+			wxString langCode = stream->GetLanguageCode();
+			if (langCode.length()) {
+				while (pgcs.GetAudioLangCodes().size() < audioStreamIdx + 1)
+					pgcs.GetAudioLangCodes().push_back(s_config.GetDefAudioLanguage());
+				pgcs.GetAudioLangCodes()[audioStreamIdx] = langCode;
+			}
 			audioStreamIdx++;
 		} else if (stream->GetType() == stSUBTITLE) {
 			stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("dvdsub") ? sfCOPY : sfNONE);
+			if (stream->GetSubtitleFormat() != sfNONE) {
+				// set language
+				wxString langCode = stream->GetLanguageCode();
+				if (langCode.length()) {
+					while (pgcs.GetSubPictures().size() < subtitleStreamIdx + 1)
+						pgcs.GetSubPictures().push_back(new SubPicture(s_config.GetDefSubtitleLanguage()));
+					pgcs.GetSubPictures()[subtitleStreamIdx]->SetLangCode(langCode);
+				}
+				subtitleStreamIdx++;
+			}
 		}
 	}
 	
@@ -632,39 +700,8 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi,
 	UpdateItems(&pdlg);
 	SendDvdChangedEvent();
 	
-	// check if video fit on DVD without re-encoding
-	bool isCopy = false;
-	for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++) {
-		Titleset* ts = m_dvd->GetTitlesets()[tsi];
-		for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++) {
-			Pgc* pgc = ts->GetTitles()[pgci];
-			for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++) {
-				Stream* stream = pgc->GetVobs()[vobi]->GetVideoStream();
-				if (stream != NULL && stream->GetVideoFormat() == vfCOPY) {
-					isCopy = true;
-					break;
-				}
-			}
-		}
-	}
-	if (isCopy && m_dvd->GetCapacity() != dcUNLIMITED && m_dvd->GetSize() > m_dvd->GetCapacityValue()) {
-		if (wxMessageBox(_("The selected video files don't fit on DVD without re-encoding.\n\
-Do you want to enable re-encoding?"), _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxYES) {
-			m_reEncodingByDefault = true;
-			for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++) {
-				Titleset* ts = m_dvd->GetTitlesets()[tsi];
-				for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++) {
-					Pgc* pgc = ts->GetTitles()[pgci];
-					for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++) {
-						Stream* stream = pgc->GetVobs()[vobi]->GetVideoStream();
-						if (stream != NULL)
-							stream->SetDestinationFormat(pgcs.GetVideo().GetFormat());
-					}
-				}
-			}
-			SendDvdChangedEvent();
-		}
-	} else if (videoBitrateOld >= 2000 && m_dvd->GetVideoBitrate() < 2000) {
+	if (!AskForReencoding(pgcs.GetVideo().GetFormat())
+			&& videoBitrateOld >= 2000 && m_dvd->GetVideoBitrate() < 2000) {
 		wxMessageBox(_("The video bitrate is reduced below 2 MB/s to fit your video on DVD. \
 This can produce DVD with bad video quality."), wxT("DVDStyler"), wxOK | wxICON_WARNING, this);
 	}
@@ -693,8 +730,8 @@ bool TitlesetManager::AddAudio(const wxString& fname) {
 		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 	else if (stream->GetSourceSampleRate() != 48000)
 		stream->SetDestinationFormat(stream->GetSourceAudioFormat());
-	if (s_config.GetDefAudio51() && m_dvd->GetAudioFormat() == afAC3
-			&& stream->GetSourceChannelNumber() != 6) {
+	if (s_config.GetDefAudio51() && m_dvd->GetAudioFormat() >= afAC3
+			&& stream->GetSourceChannelNumber() < 6) {
 		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 		stream->SetChannelNumber(6);
 		vob->SetDoNotTranscode(false);
