@@ -3,7 +3,7 @@
 // Purpose:     The video properties dialog
 // Author:      Alex Thuering
 // Created:     09.09.2010
-// RCS-ID:      $Id: VideoPropDlg.h,v 1.12 2013/06/30 15:06:45 ntalex Exp $
+// RCS-ID:      $Id: VideoPropDlg.h,v 1.13 2014/09/27 06:00:25 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -49,7 +49,6 @@ private:
 	wxCheckBox* m_keepAspectCtrl;
 	wxChoice* m_interlacedChoice;
 	wxSlider* m_slider;
-	wxStaticText* StaticText2;
 	wxSpinButton* m_frameSpinBt;
 	wxSpinCtrl* m_cropBottom;
 	wxStaticBoxSizer* m_padBox;
@@ -70,9 +69,9 @@ private:
 	wxBitmapButton* m_endBt;
 	wxChoice* m_aspectChoice;
 	wxStaticBoxSizer* m_cropBox;
-	wxStaticText* StaticText4;
 	wxSpinCtrl* m_padTop;
 	wxSpinCtrl* m_padBottom;
+	wxChoice* m_keepAspectChoice;
 	wxStaticText* m_srcText;
 	wxSpinButton* m_endFrameSpin;
 	wxSpinCtrl* m_cropRight;
@@ -91,6 +90,7 @@ private:
 	static const long INTERLACED_CHOICE_ID;
 	static const long FF_CHOICE_ID;
 	static const long KEEP_ASPECT_CB_ID;
+	static const long KEEP_ASPECT_CHOICE_ID;
 	static const long ID_SPINCTRL1;
 	static const long ID_SPINCTRL4;
 	static const long ID_SPINCTRL3;
@@ -99,8 +99,6 @@ private:
 	static const long ID_SPINCTRL6;
 	static const long ID_SPINCTRL7;
 	static const long ID_SPINCTRL8;
-	static const long ID_STATICTEXT1;
-	static const long ID_STATICTEXT5;
 	static const long ID_SPINCTRL9;
 	static const long ID_SPINCTRL10;
 	static const long ID_TEXTCTRL1;
@@ -123,9 +121,10 @@ private:
 	Stream* m_stream;
 	AspectRatio m_aspectRatio;
 	VideoFormat m_lastFormat;
+	long m_videoPos;
 	vector<int> GetPad();
 	vector<int> GetCrop();
-	void UpdatePad();
+	void UpdatePadCrop();
 	void SeekVideo(long pos, bool updateTimeCtrl = true);
 	//(*Handlers(VideoPropDlg)
 	void OnChangeFormat(wxCommandEvent& event);
