@@ -3,7 +3,7 @@
 // Purpose:     The class to store a VOB data
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: Vob.cpp,v 1.7 2015/01/06 09:48:36 ntalex Exp $
+// RCS-ID:      $Id: Vob.cpp,v 1.15 2016/12/17 17:27:38 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -123,6 +123,9 @@ bool Vob::SetFilename(const wxString& filename) {
 			stream->SetSourceAspectRatio(ffmpeg.GetFrameAspectRatio());
 			stream->SetSourceFps(ffmpeg.GetFps());
 		}
+		map<wxString, wxString> metadata = ffmpeg.GetMetadata(stIdx);
+		if (metadata.find(wxT("language")) != metadata.end())
+			stream->SetLanguage(metadata[wxT("language")]);
 		m_streams.insert(m_streams.begin() + stIdx, stream);
 	}
 	// get video duration
@@ -131,6 +134,10 @@ bool Vob::SetFilename(const wxString& filename) {
 		if (ffmpeg.SetPosition(360000, false))
 			m_duration = ffmpeg.GetPosition();
 	}
+	// set chapters
+	for (double t : ffmpeg.GetChapters()) {
+		m_cells.push_back(new Cell(lround(t*1000)));
+	}
 	return true;
 }
 
@@ -325,7 +332,7 @@ bool Vob::CalcPad(int& padx, int& pady, VideoFormat videoFormat, AspectRatio asp
 		int cropY = crop[2] + crop[3];
 		if (cropX + cropY > 0 && oldFrameSize.x > cropX && oldFrameSize.y > cropY)
 			oldAspect *= ((double) oldFrameSize.y)/oldFrameSize.x*(oldFrameSize.x - cropX)/(oldFrameSize.y - cropY);
-		float aspect = GetFrameAspectRatio(videoFormat, aspectRatio);
+		float aspect = GetFrameAspectRatio(aspectRatio);
 		wxSize frameSize = GetFrameSize(videoFormat);
 		if (aspect > oldAspect)
 			padx = lround(((double)frameSize.x)*(1.0 - oldAspect/aspect)/2);
@@ -345,7 +352,7 @@ bool Vob::CalcCrop(int& cropx, int& cropy, VideoFormat videoFormat, AspectRatio
 	if (videoFormat != vfCOPY && aspectRatio != arAUTO) {
 		float oldAspect = videoSt->GetSourceAspectRatio();
 		wxSize oldFrameSize = videoSt->GetSourceVideoSize();
-		float aspect = GetFrameAspectRatio(videoFormat, aspectRatio);
+		float aspect = GetFrameAspectRatio(aspectRatio);
 		wxSize frameSize = GetFrameSize(videoFormat);
 		int padX = pad[0] + pad[1];
 		int padY = pad[2] + pad[3];
@@ -398,16 +405,13 @@ wxString Vob::GetAllVideoFilters() {
 	if (GetFadeIn() > 0) {
 		if (result.length())
 			result += wxT(',');
-		double fps = isNTSC(videoFormat) ? 30000/1001 : 25;
-		result += wxString::Format(wxT("fade=in:%d:%d"), 0, (int) (GetFadeIn() * fps));
+		result += wxString::Format(wxT("fade=in:st=%g:d=%g"), GetStartTime(), GetFadeIn());
 	}
 	if (GetFadeOut() > 0) {
 		if (result.length())
 			result += wxT(',');
-		double fps = isNTSC(videoFormat) ? 30000/1001 : 25;
-		double endTime = GetRecordingTime() >= 0 ? GetRecordingTime() : GetDuration();
-		long startFrame = (endTime - GetFadeOut()) * fps;
-		result += wxString::Format(wxT("fade=out:%d:%d"), startFrame, (int) (GetFadeOut() * fps));
+		double endTime = GetStartTime() + GetResultDuration();
+		result += wxString::Format(wxT("fade=out:st=%g:d=%g"), endTime - GetFadeOut(), GetFadeOut());
 	}
 	
 	// add custom filters 
@@ -443,7 +447,7 @@ int Vob::GetVideoStreamIndex() {
 	return -1;
 }
 
-wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
+wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd, int nextTitle) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("vob"));
 	wxString fname = GetFilename();
 	if (type == DVDAUTHOR_XML) {
@@ -508,9 +512,9 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			node->AddProperty(wxT("recordingTime"), wxString::Format(wxT("%f"), GetRecordingTime()));
 		// video filters
 		if (GetFadeIn() > 0)
-			node->AddProperty(wxT("fadeIn"), wxString::Format(wxT("%d"), GetFadeIn()));
+			node->AddProperty(wxT("fadeIn"), wxString::Format(wxT("%g"), GetFadeIn()));
 		if (GetFadeOut() > 0)
-			node->AddProperty(wxT("fadeOut"), wxString::Format(wxT("%d"), GetFadeOut()));
+			node->AddProperty(wxT("fadeOut"), wxString::Format(wxT("%g"), GetFadeOut()));
 		if (GetVideoFilters().length() > 0)
 			node->AddProperty(wxT("videoFilters"), GetVideoFilters());
 	}
@@ -524,7 +528,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			cells = true;
 	}
 	for (vector<Cell*>::iterator it = m_cells.begin(); it != m_cells.end(); it++) {
-		if ((!(*it)->IsChapter() && m_cells.size() > 0)
+		if ((!(*it)->IsChapter() && m_cells.size() > 0) || (*it)->GetEnd() != -1
 				|| (*it)->GetPause() != 0 || (*it)->GetCommands().length() > 0) {
 			cells = true;
 			break;
@@ -535,7 +539,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			Cell* cell = *it;
 			long nextTime = it + 1 != m_cells.end() ? (*(it + 1))->GetStart() : -1;
 			long endTime = type == DVDAUTHOR_XML && cell->GetEnd() == -1 ? nextTime : cell->GetEnd();
-			node->AddChild(cell->GetXML(type, endTime));
+			node->AddChild(cell->GetXML(type, endTime, dvd, nextTitle));
 		}
 	} else {
 		wxString chapters = GetChapters();
@@ -689,8 +693,8 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu, boo
 	}
 	m_startTime = node->GetPropVal(wxT("startTime"), &val) && val.ToDouble(&dval) ? dval : 0;
 	m_recordingTime = node->GetPropVal(wxT("recordingTime"), &val) && val.ToDouble(&dval) ? dval : -1;
-	m_fadeIn = node->GetPropVal(wxT("fadeIn"), &val) && val.ToLong(&lval) ? lval : 0;
-	m_fadeOut = node->GetPropVal(wxT("fadeOut"), &val) && val.ToLong(&lval) ? lval : 0;
+	m_fadeIn = node->GetPropVal(wxT("fadeIn"), &val) && val.ToDouble(&dval) ? dval : 0;
+	m_fadeOut = node->GetPropVal(wxT("fadeOut"), &val) && val.ToDouble(&dval) ? dval : 0;
 	m_videoFilters = node->GetPropVal(wxT("videoFilters"), wxT(""));
 		
 	if (node->GetPropVal(wxT("doNotTranscode"), &val) && val == wxT("1"))
@@ -746,12 +750,17 @@ int Vob::GetOutputFileSize(DVD* dvd, int fileIdx) {
 		double duration = GetMenu()->GetSVG()->GetDuration();
 		if (duration < 1)
 			duration = s_config.GetMenuFrameCount()/(GetMenu()->GetVideoFormat() == vfPAL ? 25 : 30);
-		long menuSize = (long) (duration*s_config.GetMenuVideoBitrate()/8/3);
+		long menuSize = (long) (duration*s_config.GetMenuVideoBitrate()/8);
+		if (!s_config.GetMenuVideoCBR())
+			menuSize = menuSize / 3;
 		if (menuSize < 100)
 			menuSize = 100;
 		size += menuSize;
 	} else if (GetSlideshow()) {
-		size += GetSlideshow()->GetResultDuration()*s_config.GetSlideshowVideoBitrate()/8/3;
+		long slideshowSize = GetSlideshow()->GetResultDuration()*s_config.GetSlideshowVideoBitrate()/8;
+		if (!s_config.GetSlideshowVideoCBR())
+			slideshowSize = slideshowSize / 3;
+		size += slideshowSize;
 	} else {
 		if (m_doNotTranscode) {
 			size += GetFileSize(GetFilename());
@@ -783,8 +792,9 @@ int Vob::GetOutputFileSize(DVD* dvd, int fileIdx) {
 						break;
 					}
 				}
-				if (copyVideo)
-					size += GetFileSize(GetFilename()) - audioSize;
+				if (copyVideo) {
+					size += (GetFileSize(GetFilename()) - audioSize)*GetResultDuration()/GetDuration();
+				}
 			}
 		}
 	}
@@ -829,7 +839,7 @@ int Vob::GetRequiredSize(DVD* dvd, Cache* cache) {
 	int size = GetSize(dvd);
 	if (GetMenu() || GetSlideshow() || GetSubtitles().Count() > 0
 			 || ((!GetDoNotTranscode() || GetAudioFilenames().Count() > 0)
-					 && cache->Find(this, dvd, false).length() == 0))
+					 && cache->Find(this, dvd).length() == 0))
 		size = size * 2;
 	return size;
 }
