@@ -3,7 +3,7 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.cpp,v 1.23 2012/07/24 17:59:30 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.cpp,v 1.32 2016/12/17 17:27:38 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -15,6 +15,7 @@
 #include "SubtitlePropDlg.h"
 #include "TitlePropDlg.h"
 #include <wx/filedlg.h>
+#include <wx/progdlg.h>
 #include <wx/arrimpl.cpp>
 #include <wxSVG/mediadec_ffmpeg.h>
 #include <wxVillaLib/utils.h>
@@ -35,7 +36,9 @@ END_EVENT_TABLE()
 const int ITEM_HEIGHT = 50;
 #define ITEM_FONT wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT)
 
-VobListBox::VobListBox(TitlePropDlg* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd): wxVListBox(parent, id) {
+VobListBox::VobListBox(wxScrolledWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd,
+		TitlePropDlg* titlePropDlg): wxVListBox(parent, id) {
+	m_titlePropDlg = titlePropDlg;
 	m_vob = vob;
 	m_pgcs = pgcs;
 	m_dvd = dvd;
@@ -44,7 +47,13 @@ VobListBox::VobListBox(TitlePropDlg* parent, wxWindowID id, Vob* vob, PgcArray*
 	m_audioImg = Scale(wxBITMAP_FROM_MEMORY(audio).ConvertToImage());
 	m_subtitleImg = Scale(wxBITMAP_FROM_MEMORY(subtitle).ConvertToImage());
 	RefreshInfo();
-	SetMinSize(wxSize(-1, 300));
+	int maxh = 300;
+	for (unsigned int i = 0; i < m_infoRect.size(); i++) {
+		int h = OnMeasureItem(i);
+		if (h > maxh)
+			maxh = h;
+	}
+	SetMinSize(wxSize(-1, maxh));
 }
 
 wxCoord VobListBox::OnMeasureItem(size_t n) const {
@@ -111,8 +120,8 @@ void VobListBox::RefreshInfo() {
 		}
 		
 		// stream info
-		int stIdx = n == 0 ? 0 : (int)m_vob->GetStreams().GetCount() - m_vob->GetAudioFilenames().GetCount() + n - 1;
-		int streamsCount = n == 0 ? (int)m_vob->GetStreams().GetCount() - m_vob->GetAudioFilenames().GetCount() : 1;
+		int stIdx = n == 0 ? 0 : (int)m_vob->GetStreams().size() - m_vob->GetAudioFilenames().size() + n - 1;
+		int streamsCount = n == 0 ? (int)m_vob->GetStreams().size() - m_vob->GetAudioFilenames().size() : 1;
 		for (int stN = 0; stN < streamsCount; stN++) {
 			Stream* stream = m_vob->GetStreams()[stIdx + stN];
 			wxString srcFormat = stream->GetSourceFormat();
@@ -123,12 +132,13 @@ void VobListBox::RefreshInfo() {
 				wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
 				int top = rect.GetTop();
 				int x = rect.GetRight() + 4;
-				AddChoiceCtrl(DVD::GetVideoFormatLabels(true), stream->GetVideoFormat() - 1, x, itemY + top,
-						choiceIdx, !m_vob->GetDoNotTranscode());
+				bool copyEnabled = stream->IsCopyPossible();
+				AddChoiceCtrl(DVD::GetVideoFormatLabels(copyEnabled), stream->GetVideoFormat() - (copyEnabled ? 1 : 2),
+						x, itemY + top, choiceIdx, !m_vob->GetDoNotTranscode());
 				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
 				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
 				// button
-				AddButton(x, itemY + top, buttonIdx, true);
+				AddButton(x, itemY + top, buttonIdx, true, stIdx + stN);
 				break;
 			}
 			case stAUDIO: {
@@ -141,17 +151,20 @@ void VobListBox::RefreshInfo() {
 				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
 				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
 				// langCode
-				wxString langCode = s_config.GetDefAudioLanguage();
+				wxString langCode = stream->GetLanguageCode();
 				if (stream->GetAudioFormat() != afNONE) {
 					if (audioIdx < m_pgcs->GetAudioLangCodes().size())
 						langCode = m_pgcs->GetAudioLangCodes()[audioIdx];
 					audioIdx++;
+				} 
+				if (langCode.length() == 0) {
+					langCode = s_config.GetDefAudioLanguage();
 				}
 				int langIdx = DVD::GetAudioLanguageCodes().Index(langCode);
 				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, x, itemY + top, choiceIdx, true);
 				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
 				// button
-				AddButton(x, itemY + top, buttonIdx, true);
+				AddButton(x, itemY + top, buttonIdx, true, stIdx + stN);
 				break;
 			}
 			case stSUBTITLE: {
@@ -164,17 +177,18 @@ void VobListBox::RefreshInfo() {
 				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
 				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
 				// lang code
-				wxString langCode = s_config.GetDefSubtitleLanguage();
+				wxString langCode = stream->GetLanguageCode();
 				if (stream->GetSubtitleFormat() != sfNONE) {
 					if (subtitleIdx < m_pgcs->GetSubPictures().size())
 						langCode = m_pgcs->GetSubPictures()[subtitleIdx]->GetLangCode();
 					subtitleIdx++;
+				} 
+				if (langCode.length() == 0) {
+					langCode = s_config.GetDefSubtitleLanguage();
 				}
 				int langIdx = DVD::GetAudioLanguageCodes().Index(langCode);
 				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, x, itemY + top, choiceIdx, true);
 				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
-				// button
-				AddButton(x, itemY + top, buttonIdx, true);
 				break;
 			}
 			default:
@@ -213,7 +227,7 @@ void VobListBox::RefreshInfo() {
 		y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
 		x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
 		// button
-		AddButton(x, itemY + top, buttonIdx, true);
+		AddButton(x, itemY + top, buttonIdx, true, m_vob->GetStreams().size() + si);
 		
 		itemY += y > ITEM_HEIGHT ? y + 1 : ITEM_HEIGHT + 1;
 	}
@@ -259,17 +273,22 @@ int VobListBox::UpdateRect(wxRect& rect, wxControl* ctrl) {
 }
 
 /** Creates button and adds it to the list m_buttonList */
-void VobListBox::AddButton(int x, int y, int& buttonIdx, bool enabled) {
+void VobListBox::AddButton(int x, int y, int& buttonIdx, bool enabled, int streamIdx) {
 	wxButton* ctrl = NULL;
 	if (buttonIdx >= (int) m_buttonList.size() || m_buttonList[buttonIdx] == NULL) {
 		int h = (*m_choiceList.begin())->GetSize().GetHeight();
 		ctrl = new wxButton(this, wxID_ANY, wxT("..."), wxDefaultPosition, wxSize(h, h));
-		if (buttonIdx >= (int) m_buttonList.size())
+		if (buttonIdx >= (int) m_buttonList.size()) {
 			m_buttonList.push_back(ctrl);
-		else
+			m_buttonStreamList.push_back(streamIdx);
+		} else {
 			m_buttonList[buttonIdx] = ctrl;
-	} else
+			m_buttonStreamList[buttonIdx] = streamIdx;
+		}
+	} else {
 		ctrl = m_buttonList[buttonIdx];
+		m_buttonStreamList[buttonIdx] = streamIdx;
+	}
 	buttonIdx++;
 	ctrl->Enable(enabled);
 	ctrl->SetPosition(wxPoint(x, y));
@@ -313,9 +332,12 @@ void VobListBox::RemoveItem(int index) {
 		int choiceIdx = m_subtitleChoiceIdx + subtitleIdx;
 		m_choiceList[choiceIdx]->Hide();
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
-		int buttonIdx = m_vob->GetStreams().size() + subtitleIdx;
-		m_buttonList[buttonIdx]->Hide();
-		m_buttonList.erase(m_buttonList.begin() + buttonIdx);
+		int buttonIdx = GetButtonIdx(m_vob->GetStreams().size() + subtitleIdx);
+		if (buttonIdx >= 0) {
+			m_buttonList[buttonIdx]->Hide();
+			m_buttonList.erase(m_buttonList.begin() + buttonIdx);
+			m_buttonStreamList.erase(m_buttonStreamList.begin() + buttonIdx);
+		}
 	} else {
 		// removed audio file
 		int audioIdx = index - 1;
@@ -326,9 +348,12 @@ void VobListBox::RemoveItem(int index) {
 		m_choiceList[choiceIdx + 1]->Hide();
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
-		int buttonIdx = m_vob->GetStreams().size() - m_vob->GetAudioFilenames().size() + audioIdx;
-		m_buttonList[buttonIdx]->Hide();
-		m_buttonList.erase(m_buttonList.begin() + buttonIdx);
+		int buttonIdx = GetButtonIdx(m_vob->GetStreams().size() - m_vob->GetAudioFilenames().size() + audioIdx);
+		if (buttonIdx >= 0) {
+			m_buttonList[buttonIdx]->Hide();
+			m_buttonList.erase(m_buttonList.begin() + buttonIdx);
+			m_buttonStreamList.erase(m_buttonStreamList.begin() + buttonIdx);
+		}
 	}
 	RefreshInfo();
 	RefreshAll();
@@ -338,8 +363,8 @@ void VobListBox::AddAudio(wxString filename) {
 	m_vob->AddAudioFile(filename);
 	m_vob->SetDoNotTranscode(false);
 	// check if reencoding is needed
-	Stream* stream = m_vob->GetStreams()[m_vob->GetStreams().GetCount() - 1];
-	if (m_vob->GetStreams().GetCount() == 2 && stream->GetSourceAudioFormat() != m_dvd->GetAudioFormat())
+	Stream* stream = m_vob->GetStreams()[m_vob->GetStreams().size() - 1];
+	if (m_vob->GetStreams().size() == 2 && stream->GetSourceAudioFormat() != m_dvd->GetAudioFormat())
 		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 	else if (stream->GetSourceAudioFormat() != afMP2 && stream->GetSourceAudioFormat() != afAC3)
 		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
@@ -354,7 +379,7 @@ void VobListBox::AddAudio(wxString filename) {
 }
 
 void VobListBox::AddSubtitle(wxString filename) {
-	m_vob->GetSubtitles().Add(new TextSub(filename));
+	m_vob->AddSubtitlesFile(filename);
 	// update list box
 	RefreshInfo();
 	RefreshAll();
@@ -401,7 +426,8 @@ void VobListBox::SetValues() {
 		Stream* stream = m_vob->GetStreams()[stIdx];
 		switch (stream->GetType()) {
 		case stVIDEO:
-			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection() + 1);
+			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection()
+					+ (stream->IsCopyPossible() ? 1 : 2));
 			if (stream->GetVideoFormat() != vfCOPY)
 				m_vob->SetDoNotTranscode(false);
 			break;
@@ -464,6 +490,16 @@ int VobListBox::GetChoiceIdx(unsigned int streamIdx) {
 	return choiceIdx;
 }
 
+/** Returns index of button for given stream */
+int VobListBox::GetButtonIdx(unsigned int streamIdx) {
+	for (unsigned int btIdx = 0; btIdx < m_buttonStreamList.size(); btIdx++) {
+		if (m_buttonStreamList[btIdx] == (int) streamIdx) {
+			return btIdx;
+		}
+	}
+	return -1;
+}
+
 /** Get video format */
 int VobListBox::GetVideoFormat() {
 	return m_choiceList[m_videoChoiceIdx]->GetSelection();
@@ -513,7 +549,7 @@ void VobListBox::SetSubtitleLangCode(int subtitleIndex, wxString langCode) {
  */
 void VobListBox::ShowPropDialog() {
 	if (GetSelection() == 0) {
-		for (unsigned int i = 0; i < m_vob->GetStreams().GetCount(); i++)
+		for (unsigned int i = 0; i < m_vob->GetStreams().size(); i++)
 			if (m_vob->GetStreams()[i]->GetType() == stVIDEO)
 				ShowPropDialog(i);
 	} else if (GetSelection() < 1 + (int) m_vob->GetAudioFilenames().GetCount()) {
@@ -536,29 +572,47 @@ void VobListBox::ShowPropDialog(unsigned int streamIdx) {
 		if (idx >= m_vob->GetSubtitles().size())
 			return;
 		TextSub* textsub = m_vob->GetSubtitles()[idx];
+		
+		if (!TextSub::IsFontMapInitialized()) {
+			wxProgressDialog pDlg(wxT("DVDStyler"), _("Please wait..."), 99, this,
+					wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH);
+			pDlg.Show();
+			pDlg.SetFocus();
+			pDlg.Pulse();
+			TextSub::GetFontMap();
+			pDlg.Hide();
+		}
+		
 		SubtitlePropDlg dialog(this, textsub, GetSubtitleLangCode(idx));
 		if (dialog.ShowModal() == wxID_OK) {
 			SetSubtitleLangCode(idx, dialog.GetLangCode());
 		}
-	} else if (m_vob->GetStreams()[streamIdx]->GetType() == stVIDEO) {
-		// video
-		SetValues(); // update destination format
-		VideoPropDlg dialog(this, m_vob, m_aspectRatio);
-		if (dialog.ShowModal() == wxID_OK) {
-			SetVideoFormat(dialog.GetVideoFormat() - 1);
-			m_aspectRatio = dialog.GetAspectRatio();
-			UpdateDoNotTranscodeCheck();
-		}
-	} else if (m_vob->GetStreams()[streamIdx]->GetType() == stAUDIO) {
-		// audio
-		SetValues(); // update destination format
-		int audioIdx = streamIdx - (m_vob->GetStreams().size() - m_vob->GetAudioFilenames().GetCount());
-		wxString audioFile = audioIdx >= 0 ? m_vob->GetAudioFilenames()[audioIdx] : wxT("");
-		AudioPropDlg dialog(this, m_vob, audioFile, GetAudioLangCode(streamIdx), streamIdx);
-		if (dialog.ShowModal() == wxID_OK) {
-			SetAudioFormat(streamIdx, dialog.GetAudioFormat());
-			SetAudioLangCode(streamIdx, dialog.GetLangCode());
-			UpdateDoNotTranscodeCheck();
+	} else {
+		Stream* stream = m_vob->GetStreams()[streamIdx];
+		if (stream->GetType() == stVIDEO) {
+			// video
+			SetValues(); // update destination format
+			if (!m_titlePropDlg->ApplyChaptersCtrl())
+				return;
+			VideoPropDlg dialog(this, m_vob, m_aspectRatio);
+			if (dialog.ShowModal() == wxID_OK) {
+				bool copyEnabled = stream->IsCopyPossible();
+				SetVideoFormat(dialog.GetVideoFormat() - (copyEnabled ? 1 : 2));
+				m_aspectRatio = dialog.GetAspectRatio();
+				UpdateDoNotTranscodeCheck();
+				m_titlePropDlg->UpdateChaptersCtrl();
+			}
+		} else if (stream->GetType() == stAUDIO) {
+			// audio
+			SetValues(); // update destination format
+			int audioIdx = streamIdx - (m_vob->GetStreams().size() - m_vob->GetAudioFilenames().GetCount());
+			wxString audioFile = audioIdx >= 0 ? m_vob->GetAudioFilenames()[audioIdx] : wxT("");
+			AudioPropDlg dialog(this, m_vob, audioFile, GetAudioLangCode(streamIdx), streamIdx);
+			if (dialog.ShowModal() == wxID_OK) {
+				SetAudioFormat(streamIdx, dialog.GetAudioFormat());
+				SetAudioLangCode(streamIdx, dialog.GetLangCode());
+				UpdateDoNotTranscodeCheck();
+			}
 		}
 	}
 }
@@ -570,20 +624,20 @@ void VobListBox::OnDoubleClick(wxMouseEvent& evt) {
 
 /** Processes a button event */
 void VobListBox::OnButton(wxCommandEvent& event) {
-	int streamIdx = 0;
+	int buttonIdx = 0;
 	for (vector<wxButton*>::iterator btIt = m_buttonList.begin(); btIt < m_buttonList.end(); btIt++) {
 		if (*btIt == event.GetEventObject())
 			break;
-		streamIdx++;
+		buttonIdx++;
 	}
-	ShowPropDialog(streamIdx);
+	ShowPropDialog(m_buttonStreamList[buttonIdx]);
 }
 
 void VobListBox::UpdateDoNotTranscodeCheck() {
 	if (m_vob->GetDoNotTranscode()) {
 		SetValues(); // update destination format
 		if (!m_vob->GetDoNotTranscode())
-			((TitlePropDlg*) GetParent())->UpdateDoNotTranscodeCheck();
+			m_titlePropDlg->UpdateDoNotTranscodeCheck();
 	}
 }
 
