@@ -3,7 +3,7 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.h,v 1.12 2012/06/17 12:06:38 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.h,v 1.15 2016/05/11 20:02:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -31,7 +31,7 @@ public:
 	/**
 	 * Constructor
 	 */
-	VobListBox(TitlePropDlg* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd);
+	VobListBox(wxScrolledWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd, TitlePropDlg* titlePropDlg);
 	
 	/**
 	 * Returns the height of the specified item (in pixels)
@@ -89,6 +89,8 @@ public:
 	void ShowPropDialog();
 	
 private:
+	/** TitlePropDlg */
+	TitlePropDlg* m_titlePropDlg;
 	/** The DVD */
 	DVD* m_dvd;
 	/** Copy of VOB object to edit */
@@ -103,8 +105,10 @@ private:
 	RectListOfList m_infoRect;
 	/** The list of choice controls */
 	vector<wxChoice*> m_choiceList;
-	/** The list of choice controls */
+	/** The list of buttons */
 	vector<wxButton*> m_buttonList;
+	/** The list of stream indexes for buttons */
+	vector<int> m_buttonStreamList;
 	/** Index of first choice for audio file */
 	int m_videoChoiceIdx;
 	/** Index of first choice for audio file */
@@ -136,9 +140,11 @@ private:
 	/** Increase y and height if height of control is greater that actual height */
 	int UpdateRect(wxRect& rect, wxControl* ctrl);
 	/** Creates button and adds it to the list m_buttonList */
-	void AddButton(int x, int y, int& buttonIdx, bool enabled);
+	void AddButton(int x, int y, int& buttonIdx, bool enabled, int streamIdx);
 	/** Returns index of choice control for given stream */
 	int GetChoiceIdx(unsigned int streamIdx);
+	/** Returns index of button for given stream */
+	int GetButtonIdx(unsigned int streamIdx);
 	/** Sets video format */
 	void SetVideoFormat(int videoFormat);
 	/** Get video format */
